
#ifndef VTKPVVTKEXTENSIONSFILTERSGENERALMPI_EXPORT_H
#define VTKPVVTKEXTENSIONSFILTERSGENERALMPI_EXPORT_H

#ifdef VTKPVVTKEXTENSIONSFILTERSGENERALMPI_STATIC_DEFINE
#  define VTKPVVTKEXTENSIONSFILTERSGENERALMPI_EXPORT
#  define VTKPVVTKEXTENSIONSFILTERSGENERALMPI_NO_EXPORT
#else
#  ifndef VTKPVVTKEXTENSIONSFILTERSGENERALMPI_EXPORT
#    ifdef VTKExtensionsFiltersGeneralMPI_EXPORTS
        /* We are building this library */
#      define VTKPVVTKEXTENSIONSFILTERSGENERALMPI_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define VTKPVVTKEXTENSIONSFILTERSGENERALMPI_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef VTKPVVTKEXTENSIONSFILTERSGENERALMPI_NO_EXPORT
#    define VTKPVVTKEXTENSIONSFILTERSGENERALMPI_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef VTKPVVTKEXTENSIONSFILTERSGENERALMPI_DEPRECATED
#  define VTKPVVTKEXTENSIONSFILTERSGENERALMPI_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef VTKPVVTKEXTENSIONSFILTERSGENERALMPI_DEPRECATED_EXPORT
#  define VTKPVVTKEXTENSIONSFILTERSGENERALMPI_DEPRECATED_EXPORT VTKPVVTKEXTENSIONSFILTERSGENERALMPI_EXPORT VTKPVVTKEXTENSIONSFILTERSGENERALMPI_DEPRECATED
#endif

#ifndef VTKPVVTKEXTENSIONSFILTERSGENERALMPI_DEPRECATED_NO_EXPORT
#  define VTKPVVTKEXTENSIONSFILTERSGENERALMPI_DEPRECATED_NO_EXPORT VTKPVVTKEXTENSIONSFILTERSGENERALMPI_NO_EXPORT VTKPVVTKEXTENSIONSFILTERSGENERALMPI_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVVTKEXTENSIONSFILTERSGENERALMPI_NO_DEPRECATED
#    define VTKPVVTKEXTENSIONSFILTERSGENERALMPI_NO_DEPRECATED
#  endif
#endif

/* VTK-HeaderTest-Exclude: vtkPVVTKExtensionsFiltersGeneralMPIModule.h */

/* Include ABI Namespace */
#include "vtkABINamespace.h"

#endif /* VTKPVVTKEXTENSIONSFILTERSGENERALMPI_EXPORT_H */
