* {
    font-family: helvetica !important;
}

div.container {
    width: 100%;
    border: 1px solid gray;
}

body {
    background-color: #f9f9f9;
    margin: 0;
}

header, footer {
    font-weight: bold;
    padding-left:   1px;
    padding-right:  1px;
    padding-top:    10px;
    padding-bottom: 10px;
    color: white;
    background-color: #373737;
    clear: left;
    text-align: center;
}

footer {
    padding-top:    2px;
    padding-bottom: 2px;
}

#header h2 {
	display: inline-block;
}

#resultsArea {
	float: left;
	left: 0px;
	right: 0px;
	width: 100%;
	padding: 10px;
	background-color: white;
}

.imageArea {
	float: left;
	position: relative;
	width: 75%;
}

.sliderArea {
    font-style: normal;
    font-weight: normal;
    font-size: 1em;
    font-family: "Geneva";
    font-align: left;
	float: left;
	position: relative;
	width: 20%;
}

.cinemaImage {
	float: left;
    border: 1px solid black;
}

.flex-container {
    display: flex;
    flex-wrap: wrap;
}
