
#ifndef LEGACYGHOSTCELLSGENERATORPARALLEL_EXPORT_H
#define LEGACYGHOSTCELLSGENERATORPARALLEL_EXPORT_H

#ifdef LEGACYGHOSTCELLSGENERATORPARALLEL_STATIC_DEFINE
#  define LEGACYGHOSTCELLSGENERATORPARALLEL_EXPORT
#  define LEGACYGHOSTCELLSGENERATORPARALLEL_NO_EXPORT
#else
#  ifndef LEGACYGHOSTCELLSGENERATORPARALLEL_EXPORT
#    ifdef LegacyGhostCellsGeneratorParallel_EXPORTS
        /* We are building this library */
#      define LEGACYGHOSTCELLSGENERATORPARALLEL_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define LEGACYGHOSTCELLSGENERATORPARALLEL_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef LEGACYGHOSTCELLSGENERATORPARALLEL_NO_EXPORT
#    define LEGACYGHOSTCELLSGENERATORPARALLEL_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef LEGACYGHOSTCELLSGENERATORPARALLEL_DEPRECATED
#  define LEGACYGHOSTCELLSGENERATORPARALLEL_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef LEGACYGHOSTCELLSGENERATORPARALLEL_DEPRECATED_EXPORT
#  define LEGACYGHOSTCELLSGENERATORPARALLEL_DEPRECATED_EXPORT LEGACYGHOSTCELLSGENERATORPARALLEL_EXPORT LEGACYGHOSTCELLSGENERATORPARALLEL_DEPRECATED
#endif

#ifndef LEGACYGHOSTCELLSGENERATORPARALLEL_DEPRECATED_NO_EXPORT
#  define LEGACYGHOSTCELLSGENERATORPARALLEL_DEPRECATED_NO_EXPORT LEGACYGHOSTCELLSGENERATORPARALLEL_NO_EXPORT LEGACYGHOSTCELLSGENERATORPARALLEL_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef LEGACYGHOSTCELLSGENERATORPARALLEL_NO_DEPRECATED
#    define LEGACYGHOSTCELLSGENERATORPARALLEL_NO_DEPRECATED
#  endif
#endif

/* VTK-HeaderTest-Exclude: LegacyGhostCellsGeneratorParallelModule.h */

/* Include ABI Namespace */
#include "vtkABINamespace.h"
/* AutoInit dependencies. */
#include "LegacyGhostCellsGeneratorFiltersModule.h"
#include "vtkFiltersCoreModule.h"


/* AutoInit implementations. */
#ifdef LegacyGhostCellsGeneratorParallel_AUTOINIT_INCLUDE
#include LegacyGhostCellsGeneratorParallel_AUTOINIT_INCLUDE
#endif
#ifdef LegacyGhostCellsGeneratorParallel_AUTOINIT
#include "vtkAutoInit.h"
VTK_MODULE_AUTOINIT(LegacyGhostCellsGeneratorParallel)
#endif

#endif /* LEGACYGHOSTCELLSGENERATORPARALLEL_EXPORT_H */
