// SPDX-FileCopyrightText: Copyright (c) Kitware Inc.
// SPDX-FileCopyrightText: Copyright (c) Sandia Corporation
// SPDX-License-Identifier: BSD-3-Clause

#ifndef _QtTestingConfigure_h
#define _QtTestingConfigure_h

#define QTTESTING_BUILD_SHARED_LIBS

/* #undef QT_TESTING_WITH_PYTHON */

#define QT_TESTING_EVENT_PLAYBACK_DELAY 100

//
// This macro is used to quiet compiler warnings about unused parameters
// to methods. Only use it when the parameter really shouldn't be used.
// Don't use it as a way to shut up the compiler while you take your
// sweet time getting around to implementing the method.
//
#define pqNotUsed(x)

#endif // _QtTestingConfigure_h

