
#ifndef VTKDIGITALROCKSFILTERS_EXPORT_H
#define VTKDIGITALROCKSFILTERS_EXPORT_H

#ifdef VTKDIGITALROCKSFILTERS_STATIC_DEFINE
#  define VTKDIGITALROCKSFILTERS_EXPORT
#  define VTKDIGITALROCKSFILTERS_NO_EXPORT
#else
#  ifndef VTKDIGITALROCKSFILTERS_EXPORT
#    ifdef DigitalRocksFilters_EXPORTS
        /* We are building this library */
#      define VTKDIGITALROCKSFILTERS_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define VTKDIGITALROCKSFILTERS_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef VTKDIGITALROCKSFILTERS_NO_EXPORT
#    define VTKDIGITALROCKSFILTERS_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef VTKDIGITALROCKSFILTERS_DEPRECATED
#  define VTKDIGITALROCKSFILTERS_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef VTKDIGITALROCKSFILTERS_DEPRECATED_EXPORT
#  define VTKDIGITALROCKSFILTERS_DEPRECATED_EXPORT VTKDIGITALROCKSFILTERS_EXPORT VTKDIGITALROCKSFILTERS_DEPRECATED
#endif

#ifndef VTKDIGITALROCKSFILTERS_DEPRECATED_NO_EXPORT
#  define VTKDIGITALROCKSFILTERS_DEPRECATED_NO_EXPORT VTKDIGITALROCKSFILTERS_NO_EXPORT VTKDIGITALROCKSFILTERS_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKDIGITALROCKSFILTERS_NO_DEPRECATED
#    define VTKDIGITALROCKSFILTERS_NO_DEPRECATED
#  endif
#endif

/* VTK-HeaderTest-Exclude: vtkDigitalRocksFiltersModule.h */

/* Include ABI Namespace */
#include "vtkABINamespace.h"

#endif /* VTKDIGITALROCKSFILTERS_EXPORT_H */
