# __init__.py file for cinema_viewers

# DO NOT REMOVE THIS FILE

#
# this file is included in the repository so that this repository can
# be included as a submodule of a python module
#
# it does nothing but report the version number

Version = "2.2"

def version():
    return Version


