
#ifndef VTKPVVTKEXTENSIONSIOCGNSWRITER_EXPORT_H
#define VTKPVVTKEXTENSIONSIOCGNSWRITER_EXPORT_H

#ifdef VTKPVVTKEXTENSIONSIOCGNSWRITER_STATIC_DEFINE
#  define VTKPVVTKEXTENSIONSIOCGNSWRITER_EXPORT
#  define VTKPVVTKEXTENSIONSIOCGNSWRITER_NO_EXPORT
#else
#  ifndef VTKPVVTKEXTENSIONSIOCGNSWRITER_EXPORT
#    ifdef VTKExtensionsIOCGNSWriter_EXPORTS
        /* We are building this library */
#      define VTKPVVTKEXTENSIONSIOCGNSWRITER_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define VTKPVVTKEXTENSIONSIOCGNSWRITER_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef VTKPVVTKEXTENSIONSIOCGNSWRITER_NO_EXPORT
#    define VTKPVVTKEXTENSIONSIOCGNSWRITER_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef VTKPVVTKEXTENSIONSIOCGNSWRITER_DEPRECATED
#  define VTKPVVTKEXTENSIONSIOCGNSWRITER_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef VTKPVVTKEXTENSIONSIOCGNSWRITER_DEPRECATED_EXPORT
#  define VTKPVVTKEXTENSIONSIOCGNSWRITER_DEPRECATED_EXPORT VTKPVVTKEXTENSIONSIOCGNSWRITER_EXPORT VTKPVVTKEXTENSIONSIOCGNSWRITER_DEPRECATED
#endif

#ifndef VTKPVVTKEXTENSIONSIOCGNSWRITER_DEPRECATED_NO_EXPORT
#  define VTKPVVTKEXTENSIONSIOCGNSWRITER_DEPRECATED_NO_EXPORT VTKPVVTKEXTENSIONSIOCGNSWRITER_NO_EXPORT VTKPVVTKEXTENSIONSIOCGNSWRITER_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVVTKEXTENSIONSIOCGNSWRITER_NO_DEPRECATED
#    define VTKPVVTKEXTENSIONSIOCGNSWRITER_NO_DEPRECATED
#  endif
#endif

/* VTK-HeaderTest-Exclude: vtkPVVTKExtensionsIOCGNSWriterModule.h */

/* Include ABI Namespace */
#include "vtkABINamespace.h"

/* AutoInit implementations. */
#ifdef vtkPVVTKExtensionsIOCGNSWriter_AUTOINIT_INCLUDE
#include vtkPVVTKExtensionsIOCGNSWriter_AUTOINIT_INCLUDE
#endif
#ifdef vtkPVVTKExtensionsIOCGNSWriter_AUTOINIT
#include "vtkAutoInit.h"
VTK_MODULE_AUTOINIT(vtkPVVTKExtensionsIOCGNSWriter)
#endif

#endif /* VTKPVVTKEXTENSIONSIOCGNSWRITER_EXPORT_H */
