// SPDX-FileCopyrightText: Copyright (c) Kitware Inc.
// SPDX-License-Identifier: BSD-3-Clause

#ifndef vtkPVVersionQuick_h
#define vtkPVVersionQuick_h

#define PARAVIEW_VERSION_MAJOR 6
#define PARAVIEW_VERSION_MINOR 1
#define PARAVIEW_VERSION_EPOCH 1
#define PARAVIEW_VERSION "6.1"
#define PARAVIEW_VERSION_SHORT "6.1"

#define PARAVIEW_VERSION_CHECK(major, minor, build)                                                     \
  ((10000000000ULL * (major)) + (100000000ULL * (minor)) + (build))
#define PARAVIEW_VERSION_NUMBER_QUICK                                                                   \
  PARAVIEW_VERSION_CHECK(PARAVIEW_VERSION_MAJOR, PARAVIEW_VERSION_MINOR, PARAVIEW_VERSION_EPOCH)

#endif
