// SPDX-FileCopyrightText: Copyright (c) Kitware Inc.
// SPDX-FileCopyrightText: Copyright (c) Sandia Corporation
// SPDX-License-Identifier: BSD-3-Clause

#ifndef _pqEventTypes_h
#define _pqEventTypes_h

namespace pqEventTypes
{
enum eventType
{
  ACTION_EVENT = 0,
  CHECK_EVENT = 1
};
};

#endif // !_pqEventTypes_h
