// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: vtkPVMessage.proto
// Protobuf C++ Version: 7.35.1

#ifndef vtkPVMessage_2eproto_2epb_2eh
#define vtkPVMessage_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_bases.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_vtkPVMessage_2eproto VTKREMOTINGSERVERMANAGER_EXPORT

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct VTKREMOTINGSERVERMANAGER_EXPORT TableStruct_vtkPVMessage_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::DescriptorTable descriptor_table_vtkPVMessage_2eproto;
}  // extern "C"
namespace paraview_protobuf {
enum LinkState_LinkDescription_Direction : int;
VTKREMOTINGSERVERMANAGER_EXPORT extern const uint32_t LinkState_LinkDescription_Direction_internal_data_[];
enum MousePointer_Ratio : int;
VTKREMOTINGSERVERMANAGER_EXPORT extern const uint32_t MousePointer_Ratio_internal_data_[];
enum QtEvent_QtEventType : int;
VTKREMOTINGSERVERMANAGER_EXPORT extern const uint32_t QtEvent_QtEventType_internal_data_[];
enum Variant_Type : int;
VTKREMOTINGSERVERMANAGER_EXPORT extern const uint32_t Variant_Type_internal_data_[];
class ChartViewBounds;
struct ChartViewBoundsGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern ChartViewBoundsGlobalsTypeInternal ChartViewBounds_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull ChartViewBounds_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const ChartViewBoundsGlobalsTypeInternal ChartViewBounds_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ChatMessage;
struct ChatMessageGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern ChatMessageGlobalsTypeInternal ChatMessage_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull ChatMessage_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const ChatMessageGlobalsTypeInternal ChatMessage_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ClientsInformation;
struct ClientsInformationGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern ClientsInformationGlobalsTypeInternal ClientsInformation_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull ClientsInformation_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const ClientsInformationGlobalsTypeInternal ClientsInformation_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ClientsInformation_ClientInfo;
struct ClientsInformation_ClientInfoGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern ClientsInformation_ClientInfoGlobalsTypeInternal ClientsInformation_ClientInfo_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull ClientsInformation_ClientInfo_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const ClientsInformation_ClientInfoGlobalsTypeInternal ClientsInformation_ClientInfo_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class DefinitionHeader;
struct DefinitionHeaderGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern DefinitionHeaderGlobalsTypeInternal DefinitionHeader_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull DefinitionHeader_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const DefinitionHeaderGlobalsTypeInternal DefinitionHeader_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class LinkState;
struct LinkStateGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern LinkStateGlobalsTypeInternal LinkState_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull LinkState_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const LinkStateGlobalsTypeInternal LinkState_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class LinkState_LinkDescription;
struct LinkState_LinkDescriptionGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern LinkState_LinkDescriptionGlobalsTypeInternal LinkState_LinkDescription_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull LinkState_LinkDescription_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const LinkState_LinkDescriptionGlobalsTypeInternal LinkState_LinkDescription_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Message;
struct MessageGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern MessageGlobalsTypeInternal Message_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull Message_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const MessageGlobalsTypeInternal Message_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class MessageCollection;
struct MessageCollectionGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern MessageCollectionGlobalsTypeInternal MessageCollection_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull MessageCollection_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const MessageCollectionGlobalsTypeInternal MessageCollection_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class MousePointer;
struct MousePointerGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern MousePointerGlobalsTypeInternal MousePointer_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull MousePointer_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const MousePointerGlobalsTypeInternal MousePointer_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class PXMRegistrationState;
struct PXMRegistrationStateGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern PXMRegistrationStateGlobalsTypeInternal PXMRegistrationState_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull PXMRegistrationState_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const PXMRegistrationStateGlobalsTypeInternal PXMRegistrationState_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class PXMRegistrationState_Entry;
struct PXMRegistrationState_EntryGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern PXMRegistrationState_EntryGlobalsTypeInternal PXMRegistrationState_Entry_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull PXMRegistrationState_Entry_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const PXMRegistrationState_EntryGlobalsTypeInternal PXMRegistrationState_Entry_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ProxyDefinitionState;
struct ProxyDefinitionStateGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern ProxyDefinitionStateGlobalsTypeInternal ProxyDefinitionState_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull ProxyDefinitionState_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const ProxyDefinitionStateGlobalsTypeInternal ProxyDefinitionState_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ProxyDefinitionState_ProxyXMLDefinition;
struct ProxyDefinitionState_ProxyXMLDefinitionGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern ProxyDefinitionState_ProxyXMLDefinitionGlobalsTypeInternal ProxyDefinitionState_ProxyXMLDefinition_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull ProxyDefinitionState_ProxyXMLDefinition_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const ProxyDefinitionState_ProxyXMLDefinitionGlobalsTypeInternal ProxyDefinitionState_ProxyXMLDefinition_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ProxySelectionModelState;
struct ProxySelectionModelStateGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern ProxySelectionModelStateGlobalsTypeInternal ProxySelectionModelState_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull ProxySelectionModelState_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const ProxySelectionModelStateGlobalsTypeInternal ProxySelectionModelState_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ProxyState;
struct ProxyStateGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern ProxyStateGlobalsTypeInternal ProxyState_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull ProxyState_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const ProxyStateGlobalsTypeInternal ProxyState_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ProxyState_Annotation;
struct ProxyState_AnnotationGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern ProxyState_AnnotationGlobalsTypeInternal ProxyState_Annotation_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull ProxyState_Annotation_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const ProxyState_AnnotationGlobalsTypeInternal ProxyState_Annotation_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ProxyState_Property;
struct ProxyState_PropertyGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern ProxyState_PropertyGlobalsTypeInternal ProxyState_Property_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull ProxyState_Property_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const ProxyState_PropertyGlobalsTypeInternal ProxyState_Property_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ProxyState_SubProxy;
struct ProxyState_SubProxyGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern ProxyState_SubProxyGlobalsTypeInternal ProxyState_SubProxy_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull ProxyState_SubProxy_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const ProxyState_SubProxyGlobalsTypeInternal ProxyState_SubProxy_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ProxyState_UserData;
struct ProxyState_UserDataGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern ProxyState_UserDataGlobalsTypeInternal ProxyState_UserData_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull ProxyState_UserData_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const ProxyState_UserDataGlobalsTypeInternal ProxyState_UserData_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class PullRequest;
struct PullRequestGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern PullRequestGlobalsTypeInternal PullRequest_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull PullRequest_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const PullRequestGlobalsTypeInternal PullRequest_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class QtEvent;
struct QtEventGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern QtEventGlobalsTypeInternal QtEvent_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull QtEvent_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const QtEventGlobalsTypeInternal QtEvent_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Variant;
struct VariantGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern VariantGlobalsTypeInternal Variant_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull Variant_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const VariantGlobalsTypeInternal Variant_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class VariantList;
struct VariantListGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
VTKREMOTINGSERVERMANAGER_EXPORT extern VariantListGlobalsTypeInternal VariantList_globals_;
VTKREMOTINGSERVERMANAGER_EXPORT extern const ::google::protobuf::internal::ClassDataFull VariantList_class_data_;
#else
VTKREMOTINGSERVERMANAGER_EXPORT extern const VariantListGlobalsTypeInternal VariantList_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace paraview_protobuf
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::paraview_protobuf::LinkState_LinkDescription_Direction_internal_data_>
    internal::EnumTraitsImpl::value<::paraview_protobuf::LinkState_LinkDescription_Direction>;
template <>
internal::EnumTraitsT<::paraview_protobuf::MousePointer_Ratio_internal_data_>
    internal::EnumTraitsImpl::value<::paraview_protobuf::MousePointer_Ratio>;
template <>
internal::EnumTraitsT<::paraview_protobuf::QtEvent_QtEventType_internal_data_>
    internal::EnumTraitsImpl::value<::paraview_protobuf::QtEvent_QtEventType>;
template <>
internal::EnumTraitsT<::paraview_protobuf::Variant_Type_internal_data_>
    internal::EnumTraitsImpl::value<::paraview_protobuf::Variant_Type>;
}  // namespace protobuf
}  // namespace google

namespace paraview_protobuf {
enum Variant_Type : int {
  Variant_Type_INT = 0,
  Variant_Type_FLOAT64 = 1,
  Variant_Type_IDTYPE = 2,
  Variant_Type_STRING = 3,
  Variant_Type_PROXY = 4,
  Variant_Type_INPUT = 5,
  Variant_Type_BYTES = 6,
  Variant_Type_MAP = 7,
};

VTKREMOTINGSERVERMANAGER_EXPORT extern const uint32_t Variant_Type_internal_data_[];
inline constexpr Variant_Type Variant_Type_Type_MIN =
    static_cast<Variant_Type>(0);
inline constexpr Variant_Type Variant_Type_Type_MAX =
    static_cast<Variant_Type>(7);
[[nodiscard]] inline bool Variant_Type_IsValid(int value) {
  return 0 <= value && value <= 7;
}
inline constexpr int Variant_Type_Type_ARRAYSIZE = 7 + 1;
[[nodiscard]] VTKREMOTINGSERVERMANAGER_EXPORT const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
Variant_Type_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(Variant_Type) {
  return Variant_Type_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& Variant_Type_Name(T value) {
  static_assert(::std::is_same<T, Variant_Type>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to Type_Name().");
  return Variant_Type_Name(static_cast<Variant_Type>(value));
}
template <>
[[nodiscard]] inline const ::std::string& Variant_Type_Name(Variant_Type value) {
  return ::google::protobuf::internal::NameOfDenseEnum<Variant_Type_descriptor, 0, 7>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool Variant_Type_Parse(
    ::absl::string_view name, Variant_Type* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<Variant_Type>(Variant_Type_descriptor(), name,
                                           value);
}
enum LinkState_LinkDescription_Direction : int {
  LinkState_LinkDescription_Direction_NONE = 0,
  LinkState_LinkDescription_Direction_INPUT = 1,
  LinkState_LinkDescription_Direction_OUTPUT = 2,
};

VTKREMOTINGSERVERMANAGER_EXPORT extern const uint32_t LinkState_LinkDescription_Direction_internal_data_[];
inline constexpr LinkState_LinkDescription_Direction LinkState_LinkDescription_Direction_Direction_MIN =
    static_cast<LinkState_LinkDescription_Direction>(0);
inline constexpr LinkState_LinkDescription_Direction LinkState_LinkDescription_Direction_Direction_MAX =
    static_cast<LinkState_LinkDescription_Direction>(2);
[[nodiscard]] inline bool LinkState_LinkDescription_Direction_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int LinkState_LinkDescription_Direction_Direction_ARRAYSIZE = 2 + 1;
[[nodiscard]] VTKREMOTINGSERVERMANAGER_EXPORT const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
LinkState_LinkDescription_Direction_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(LinkState_LinkDescription_Direction) {
  return LinkState_LinkDescription_Direction_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& LinkState_LinkDescription_Direction_Name(T value) {
  static_assert(::std::is_same<T, LinkState_LinkDescription_Direction>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to Direction_Name().");
  return LinkState_LinkDescription_Direction_Name(static_cast<LinkState_LinkDescription_Direction>(value));
}
template <>
[[nodiscard]] inline const ::std::string& LinkState_LinkDescription_Direction_Name(LinkState_LinkDescription_Direction value) {
  return ::google::protobuf::internal::NameOfDenseEnum<LinkState_LinkDescription_Direction_descriptor, 0, 2>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool LinkState_LinkDescription_Direction_Parse(
    ::absl::string_view name, LinkState_LinkDescription_Direction* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<LinkState_LinkDescription_Direction>(LinkState_LinkDescription_Direction_descriptor(), name,
                                           value);
}
enum MousePointer_Ratio : int {
  MousePointer_Ratio_BOTH = 0,
  MousePointer_Ratio_HEIGHT = 1,
  MousePointer_Ratio_WIDTH = 2,
};

VTKREMOTINGSERVERMANAGER_EXPORT extern const uint32_t MousePointer_Ratio_internal_data_[];
inline constexpr MousePointer_Ratio MousePointer_Ratio_Ratio_MIN =
    static_cast<MousePointer_Ratio>(0);
inline constexpr MousePointer_Ratio MousePointer_Ratio_Ratio_MAX =
    static_cast<MousePointer_Ratio>(2);
[[nodiscard]] inline bool MousePointer_Ratio_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int MousePointer_Ratio_Ratio_ARRAYSIZE = 2 + 1;
[[nodiscard]] VTKREMOTINGSERVERMANAGER_EXPORT const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
MousePointer_Ratio_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(MousePointer_Ratio) {
  return MousePointer_Ratio_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& MousePointer_Ratio_Name(T value) {
  static_assert(::std::is_same<T, MousePointer_Ratio>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to Ratio_Name().");
  return MousePointer_Ratio_Name(static_cast<MousePointer_Ratio>(value));
}
template <>
[[nodiscard]] inline const ::std::string& MousePointer_Ratio_Name(MousePointer_Ratio value) {
  return ::google::protobuf::internal::NameOfDenseEnum<MousePointer_Ratio_descriptor, 0, 2>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool MousePointer_Ratio_Parse(
    ::absl::string_view name, MousePointer_Ratio* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<MousePointer_Ratio>(MousePointer_Ratio_descriptor(), name,
                                           value);
}
enum QtEvent_QtEventType : int {
  QtEvent_QtEventType_CHAT = 1,
  QtEvent_QtEventType_OTHER = 3,
};

VTKREMOTINGSERVERMANAGER_EXPORT extern const uint32_t QtEvent_QtEventType_internal_data_[];
inline constexpr QtEvent_QtEventType QtEvent_QtEventType_QtEventType_MIN =
    static_cast<QtEvent_QtEventType>(1);
inline constexpr QtEvent_QtEventType QtEvent_QtEventType_QtEventType_MAX =
    static_cast<QtEvent_QtEventType>(3);
[[nodiscard]] inline bool QtEvent_QtEventType_IsValid(int value) {
  return 0 <= value && value <= 3 && ((10u >> value) & 1) != 0;
}
inline constexpr int QtEvent_QtEventType_QtEventType_ARRAYSIZE = 3 + 1;
[[nodiscard]] VTKREMOTINGSERVERMANAGER_EXPORT const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
QtEvent_QtEventType_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(QtEvent_QtEventType) {
  return QtEvent_QtEventType_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& QtEvent_QtEventType_Name(T value) {
  static_assert(::std::is_same<T, QtEvent_QtEventType>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to QtEventType_Name().");
  return QtEvent_QtEventType_Name(static_cast<QtEvent_QtEventType>(value));
}
template <>
[[nodiscard]] inline const ::std::string& QtEvent_QtEventType_Name(QtEvent_QtEventType value) {
  return ::google::protobuf::internal::NameOfDenseEnum<QtEvent_QtEventType_descriptor, 1, 3>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool QtEvent_QtEventType_Parse(
    ::absl::string_view name, QtEvent_QtEventType* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<QtEvent_QtEventType>(QtEvent_QtEventType_descriptor(), name,
                                           value);
}
using ::google::protobuf::internal::generated_enum::AbslParseFlag;
using ::google::protobuf::internal::generated_enum::AbslUnparseFlag;

// ===================================================================


// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Variant final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:paraview_protobuf.Variant) */ {
 public:
  inline Variant() : Variant(nullptr) {}
  ~Variant() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Variant* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Variant));
  }
#endif

  template <typename = void>
  explicit constexpr Variant(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Variant(const Variant& from) : Variant(nullptr, from) {}
  inline Variant(Variant&& from) noexcept : Variant(nullptr, ::std::move(from)) {}
  inline Variant& operator=(const Variant& from) {
    CopyFrom(from);
    return *this;
  }
  inline Variant& operator=(Variant&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Variant& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Variant>(&Variant_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(Variant& a, Variant& b) { a.Swap(&b); }
  inline void Swap(Variant* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Variant* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Variant* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Variant>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Variant& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Variant& from) { Variant::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return IsInitializedImpl(*this);
  }

  private:
  static bool IsInitializedImpl(const MessageLite& msg);

  public:
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Variant* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.Variant"; }

  explicit Variant(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Variant(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Variant& from);
  Variant(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Variant&& from) noexcept
      : Variant(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using Type = Variant_Type;
  static constexpr Type INT = Variant_Type_INT;
  static constexpr Type FLOAT64 = Variant_Type_FLOAT64;
  static constexpr Type IDTYPE = Variant_Type_IDTYPE;
  static constexpr Type STRING = Variant_Type_STRING;
  static constexpr Type PROXY = Variant_Type_PROXY;
  static constexpr Type INPUT = Variant_Type_INPUT;
  static constexpr Type BYTES = Variant_Type_BYTES;
  static constexpr Type MAP = Variant_Type_MAP;
  [[nodiscard]] static inline bool Type_IsValid(int value) {
    return Variant_Type_IsValid(value);
  }
  static constexpr Type Type_MIN = Variant_Type_Type_MIN;
  static constexpr Type Type_MAX = Variant_Type_Type_MAX;
  static constexpr int Type_ARRAYSIZE = Variant_Type_Type_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL Type_descriptor() {
    return Variant_Type_descriptor();
  }
  template <typename T>
  [[nodiscard]] static inline const ::std::string& Type_Name(T value) {
    return Variant_Type_Name(value);
  }
  [[nodiscard]] static inline bool Type_Parse(
      ::absl::string_view name, Type* PROTOBUF_NONNULL value) {
    return Variant_Type_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  enum : int {
    kIdtypeFieldNumber = 3,
    kIntegerFieldNumber = 4,
    kFloat64FieldNumber = 5,
    kProxyGlobalIdFieldNumber = 6,
    kPortNumberFieldNumber = 7,
    kTxtFieldNumber = 8,
    kBinaryFieldNumber = 9,
    kTypeFieldNumber = 1,
  };
  // repeated sint64 idtype = 3;
  [[nodiscard]] int idtype_size()
      const;
  private:
  int _internal_idtype_size() const;

  public:
  void clear_idtype() ;
  [[nodiscard]] ::int64_t idtype(int index) const;
  void set_idtype(int index, ::int64_t value);
  void add_idtype(::int64_t value);
  [[nodiscard]] const ::google::protobuf::RepeatedField<::int64_t>& idtype()
      const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL mutable_idtype();

  private:
  const ::google::protobuf::RepeatedField<::int64_t>& _internal_idtype() const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL _internal_mutable_idtype();

  public:
  // repeated sint32 integer = 4;
  [[nodiscard]] int integer_size()
      const;
  private:
  int _internal_integer_size() const;

  public:
  void clear_integer() ;
  [[nodiscard]] ::int32_t integer(int index) const;
  void set_integer(int index, ::int32_t value);
  void add_integer(::int32_t value);
  [[nodiscard]] const ::google::protobuf::RepeatedField<::int32_t>& integer()
      const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL mutable_integer();

  private:
  const ::google::protobuf::RepeatedField<::int32_t>& _internal_integer() const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL _internal_mutable_integer();

  public:
  // repeated double float64 = 5;
  [[nodiscard]] int float64_size()
      const;
  private:
  int _internal_float64_size() const;

  public:
  void clear_float64() ;
  [[nodiscard]] double float64(int index) const;
  void set_float64(int index, double value);
  void add_float64(double value);
  [[nodiscard]] const ::google::protobuf::RepeatedField<double>& float64()
      const;
  ::google::protobuf::RepeatedField<double>* PROTOBUF_NONNULL mutable_float64();

  private:
  const ::google::protobuf::RepeatedField<double>& _internal_float64() const;
  ::google::protobuf::RepeatedField<double>* PROTOBUF_NONNULL _internal_mutable_float64();

  public:
  // repeated uint64 proxy_global_id = 6;
  [[nodiscard]] int proxy_global_id_size()
      const;
  private:
  int _internal_proxy_global_id_size() const;

  public:
  void clear_proxy_global_id() ;
  [[nodiscard]] ::uint64_t proxy_global_id(int index) const;
  void set_proxy_global_id(int index, ::uint64_t value);
  void add_proxy_global_id(::uint64_t value);
  [[nodiscard]] const ::google::protobuf::RepeatedField<::uint64_t>& proxy_global_id()
      const;
  ::google::protobuf::RepeatedField<::uint64_t>* PROTOBUF_NONNULL mutable_proxy_global_id();

  private:
  const ::google::protobuf::RepeatedField<::uint64_t>& _internal_proxy_global_id() const;
  ::google::protobuf::RepeatedField<::uint64_t>* PROTOBUF_NONNULL _internal_mutable_proxy_global_id();

  public:
  // repeated int32 port_number = 7;
  [[nodiscard]] int port_number_size()
      const;
  private:
  int _internal_port_number_size() const;

  public:
  void clear_port_number() ;
  [[nodiscard]] ::int32_t port_number(int index) const;
  void set_port_number(int index, ::int32_t value);
  void add_port_number(::int32_t value);
  [[nodiscard]] const ::google::protobuf::RepeatedField<::int32_t>& port_number()
      const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL mutable_port_number();

  private:
  const ::google::protobuf::RepeatedField<::int32_t>& _internal_port_number() const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL _internal_mutable_port_number();

  public:
  // repeated string txt = 8;
  [[nodiscard]] int txt_size()
      const;
  private:
  int _internal_txt_size() const;

  public:
  void clear_txt() ;
  [[nodiscard]] const ::std::string& txt(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_txt(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_txt(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_txt();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_txt(Arg_&& value, Args_... args);
  [[nodiscard]] const
      ::google::protobuf::RepeatedPtrField<::std::string>&
      txt() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::std::string>*
      PROTOBUF_NONNULL
      mutable_txt();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_txt() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_txt();

  public:
  // repeated bytes binary = 9;
  [[nodiscard]] int binary_size()
      const;
  private:
  int _internal_binary_size() const;

  public:
  void clear_binary() ;
  [[nodiscard]] const ::std::string& binary(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_binary(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_binary(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_binary();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_binary(Arg_&& value, Args_... args);
  [[nodiscard]] const
      ::google::protobuf::RepeatedPtrField<::std::string>&
      binary() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::std::string>*
      PROTOBUF_NONNULL
      mutable_binary();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_binary() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_binary();

  public:
  // required .paraview_protobuf.Variant.Type type = 1;
  [[nodiscard]] bool has_type()
      const;
  void clear_type() ;
  [[nodiscard]] ::paraview_protobuf::Variant_Type type() const;
  void set_type(::paraview_protobuf::Variant_Type value);

  private:
  ::paraview_protobuf::Variant_Type _internal_type() const;
  void _internal_set_type(::paraview_protobuf::Variant_Type value);

  public:
  // @@protoc_insertion_point(class_scope:paraview_protobuf.Variant)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<4, 8,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Variant& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedField<::int64_t> idtype_;
    ::google::protobuf::RepeatedField<::int32_t> integer_;
    ::google::protobuf::RepeatedField<double> float64_;
    ::google::protobuf::RepeatedField<::uint64_t> proxy_global_id_;
    ::google::protobuf::RepeatedField<::int32_t> port_number_;
    ::google::protobuf::RepeatedPtrField<::std::string> txt_;
    ::google::protobuf::RepeatedPtrField<::std::string> binary_;
    int type_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED QtEvent final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:paraview_protobuf.QtEvent) */ {
 public:
  inline QtEvent() : QtEvent(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(QtEvent* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(QtEvent));
  }
#endif

  template <typename = void>
  explicit constexpr QtEvent(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline QtEvent(const QtEvent& from) : QtEvent(nullptr, from) {}
  inline QtEvent(QtEvent&& from) noexcept : QtEvent(nullptr, ::std::move(from)) {}
  inline QtEvent& operator=(const QtEvent& from) {
    CopyFrom(from);
    return *this;
  }
  inline QtEvent& operator=(QtEvent&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const QtEvent& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<QtEvent>(&QtEvent_globals_);
  }
  static constexpr int kIndexInFileMessages = 23;
  friend void swap(QtEvent& a, QtEvent& b) { a.Swap(&b); }
  inline void Swap(QtEvent* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(QtEvent* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] QtEvent* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<QtEvent>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const QtEvent& from) { ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from); }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const QtEvent& from) { ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from); }

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.QtEvent"; }

  explicit QtEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  QtEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const QtEvent& from);
  QtEvent(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, QtEvent&& from) noexcept
      : QtEvent(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using QtEventType = QtEvent_QtEventType;
  static constexpr QtEventType CHAT = QtEvent_QtEventType_CHAT;
  static constexpr QtEventType OTHER = QtEvent_QtEventType_OTHER;
  [[nodiscard]] static inline bool QtEventType_IsValid(int value) {
    return QtEvent_QtEventType_IsValid(value);
  }
  static constexpr QtEventType QtEventType_MIN = QtEvent_QtEventType_QtEventType_MIN;
  static constexpr QtEventType QtEventType_MAX = QtEvent_QtEventType_QtEventType_MAX;
  static constexpr int QtEventType_ARRAYSIZE = QtEvent_QtEventType_QtEventType_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL QtEventType_descriptor() {
    return QtEvent_QtEventType_descriptor();
  }
  template <typename T>
  [[nodiscard]] static inline const ::std::string& QtEventType_Name(T value) {
    return QtEvent_QtEventType_Name(value);
  }
  [[nodiscard]] static inline bool QtEventType_Parse(
      ::absl::string_view name, QtEventType* PROTOBUF_NONNULL value) {
    return QtEvent_QtEventType_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  inline static constexpr int kTypeFieldNumber = 98;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::EnumTypeTraits< ::paraview_protobuf::QtEvent_QtEventType>, 14, false>(type);
  // @@protoc_insertion_point(class_scope:paraview_protobuf.QtEvent)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 0,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED PullRequest final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:paraview_protobuf.PullRequest) */ {
 public:
  inline PullRequest() : PullRequest(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PullRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PullRequest));
  }
#endif

  template <typename = void>
  explicit constexpr PullRequest(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline PullRequest(const PullRequest& from) : PullRequest(nullptr, from) {}
  inline PullRequest(PullRequest&& from) noexcept : PullRequest(nullptr, ::std::move(from)) {}
  inline PullRequest& operator=(const PullRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline PullRequest& operator=(PullRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const PullRequest& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<PullRequest>(&PullRequest_globals_);
  }
  static constexpr int kIndexInFileMessages = 14;
  friend void swap(PullRequest& a, PullRequest& b) { a.Swap(&b); }
  inline void Swap(PullRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PullRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] PullRequest* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<PullRequest>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const PullRequest& from) { ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from); }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const PullRequest& from) { ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from); }

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.PullRequest"; }

  explicit PullRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PullRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PullRequest& from);
  PullRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PullRequest&& from) noexcept
      : PullRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  inline static constexpr int kArgumentsFieldNumber = 40;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::RepeatedMessageTypeTraits< ::paraview_protobuf::Variant >, 11, false>(arguments);
  // @@protoc_insertion_point(class_scope:paraview_protobuf.PullRequest)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 0,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ProxyState_SubProxy final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:paraview_protobuf.ProxyState.SubProxy) */ {
 public:
  inline ProxyState_SubProxy() : ProxyState_SubProxy(nullptr) {}
  ~ProxyState_SubProxy() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ProxyState_SubProxy* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ProxyState_SubProxy));
  }
#endif

  template <typename = void>
  explicit constexpr ProxyState_SubProxy(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ProxyState_SubProxy(const ProxyState_SubProxy& from) : ProxyState_SubProxy(nullptr, from) {}
  inline ProxyState_SubProxy(ProxyState_SubProxy&& from) noexcept : ProxyState_SubProxy(nullptr, ::std::move(from)) {}
  inline ProxyState_SubProxy& operator=(const ProxyState_SubProxy& from) {
    CopyFrom(from);
    return *this;
  }
  inline ProxyState_SubProxy& operator=(ProxyState_SubProxy&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ProxyState_SubProxy& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ProxyState_SubProxy>(&ProxyState_SubProxy_globals_);
  }
  static constexpr int kIndexInFileMessages = 5;
  friend void swap(ProxyState_SubProxy& a, ProxyState_SubProxy& b) { a.Swap(&b); }
  inline void Swap(ProxyState_SubProxy* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ProxyState_SubProxy* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ProxyState_SubProxy* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ProxyState_SubProxy>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ProxyState_SubProxy& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ProxyState_SubProxy& from) { ProxyState_SubProxy::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return IsInitializedImpl(*this);
  }

  private:
  static bool IsInitializedImpl(const MessageLite& msg);

  public:
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ProxyState_SubProxy* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.ProxyState.SubProxy"; }

  explicit ProxyState_SubProxy(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ProxyState_SubProxy(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ProxyState_SubProxy& from);
  ProxyState_SubProxy(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ProxyState_SubProxy&& from) noexcept
      : ProxyState_SubProxy(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kNameFieldNumber = 1,
    kGlobalIdFieldNumber = 2,
  };
  // required string name = 1;
  [[nodiscard]] bool has_name()
      const;
  void clear_name() ;
  [[nodiscard]] const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // required uint32 global_id = 2;
  [[nodiscard]] bool has_global_id()
      const;
  void clear_global_id() ;
  [[nodiscard]] ::uint32_t global_id() const;
  void set_global_id(::uint32_t value);

  private:
  ::uint32_t _internal_global_id() const;
  void _internal_set_global_id(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:paraview_protobuf.ProxyState.SubProxy)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ProxyState_SubProxy& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::uint32_t global_id_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ProxyState_Annotation final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:paraview_protobuf.ProxyState.Annotation) */ {
 public:
  inline ProxyState_Annotation() : ProxyState_Annotation(nullptr) {}
  ~ProxyState_Annotation() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ProxyState_Annotation* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ProxyState_Annotation));
  }
#endif

  template <typename = void>
  explicit constexpr ProxyState_Annotation(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ProxyState_Annotation(const ProxyState_Annotation& from) : ProxyState_Annotation(nullptr, from) {}
  inline ProxyState_Annotation(ProxyState_Annotation&& from) noexcept : ProxyState_Annotation(nullptr, ::std::move(from)) {}
  inline ProxyState_Annotation& operator=(const ProxyState_Annotation& from) {
    CopyFrom(from);
    return *this;
  }
  inline ProxyState_Annotation& operator=(ProxyState_Annotation&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ProxyState_Annotation& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ProxyState_Annotation>(&ProxyState_Annotation_globals_);
  }
  static constexpr int kIndexInFileMessages = 6;
  friend void swap(ProxyState_Annotation& a, ProxyState_Annotation& b) { a.Swap(&b); }
  inline void Swap(ProxyState_Annotation* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ProxyState_Annotation* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ProxyState_Annotation* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ProxyState_Annotation>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ProxyState_Annotation& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ProxyState_Annotation& from) { ProxyState_Annotation::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return IsInitializedImpl(*this);
  }

  private:
  static bool IsInitializedImpl(const MessageLite& msg);

  public:
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ProxyState_Annotation* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.ProxyState.Annotation"; }

  explicit ProxyState_Annotation(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ProxyState_Annotation(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ProxyState_Annotation& from);
  ProxyState_Annotation(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ProxyState_Annotation&& from) noexcept
      : ProxyState_Annotation(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kKeyFieldNumber = 1,
    kValueFieldNumber = 2,
  };
  // required string key = 1;
  [[nodiscard]] bool has_key()
      const;
  void clear_key() ;
  [[nodiscard]] const ::std::string& key() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_key(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_key();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_key();
  void set_allocated_key(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_key() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_key(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_key();

  public:
  // required string value = 2;
  [[nodiscard]] bool has_value()
      const;
  void clear_value() ;
  [[nodiscard]] const ::std::string& value() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_value(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_value();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_value();
  void set_allocated_value(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_value() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_value(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_value();

  public:
  // @@protoc_insertion_point(class_scope:paraview_protobuf.ProxyState.Annotation)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ProxyState_Annotation& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr key_;
    ::google::protobuf::internal::ArenaStringPtr value_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ProxyState final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:paraview_protobuf.ProxyState) */ {
 public:
  inline ProxyState() : ProxyState(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ProxyState* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ProxyState));
  }
#endif

  template <typename = void>
  explicit constexpr ProxyState(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ProxyState(const ProxyState& from) : ProxyState(nullptr, from) {}
  inline ProxyState(ProxyState&& from) noexcept : ProxyState(nullptr, ::std::move(from)) {}
  inline ProxyState& operator=(const ProxyState& from) {
    CopyFrom(from);
    return *this;
  }
  inline ProxyState& operator=(ProxyState&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ProxyState& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ProxyState>(&ProxyState_globals_);
  }
  static constexpr int kIndexInFileMessages = 9;
  friend void swap(ProxyState& a, ProxyState& b) { a.Swap(&b); }
  inline void Swap(ProxyState* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ProxyState* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ProxyState* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<ProxyState>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const ProxyState& from) { ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from); }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const ProxyState& from) { ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from); }

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.ProxyState"; }

  explicit ProxyState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ProxyState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ProxyState& from);
  ProxyState(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ProxyState&& from) noexcept
      : ProxyState(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using SubProxy = ProxyState_SubProxy;
  using Annotation = ProxyState_Annotation;
  using UserData = ProxyState_UserData;
  using Property = ProxyState_Property;

  // accessors -------------------------------------------------------
  inline static constexpr int kXmlGroupFieldNumber = 20;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::StringTypeTraits, 12, false>(xml_group);
  inline static constexpr int kXmlNameFieldNumber = 21;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::StringTypeTraits, 12, false>(xml_name);
  inline static constexpr int kXmlSubProxyNameFieldNumber = 22;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::StringTypeTraits, 12, false>(xml_sub_proxy_name);
  inline static constexpr int kPropertyFieldNumber = 23;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::RepeatedMessageTypeTraits< ::paraview_protobuf::ProxyState_Property >, 11, false>(property);
  inline static constexpr int kSubproxyFieldNumber = 24;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::RepeatedMessageTypeTraits< ::paraview_protobuf::ProxyState_SubProxy >, 11, false>(subproxy);
  inline static constexpr int kAnnotationFieldNumber = 25;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::RepeatedMessageTypeTraits< ::paraview_protobuf::ProxyState_Annotation >, 11, false>(annotation);
  inline static constexpr int kHasAnnotationFieldNumber = 26;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::PrimitiveTypeTraits< bool >, 8, false>(has_annotation);
  inline static constexpr int kUserDataFieldNumber = 27;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::RepeatedMessageTypeTraits< ::paraview_protobuf::ProxyState_UserData >, 11, false>(user_data);
  inline static constexpr int kLogNameFieldNumber = 28;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::StringTypeTraits, 12, false>(log_name);
  inline static constexpr int kVtkClassnameFieldNumber = 29;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::StringTypeTraits, 12, false>(vtk_classname);
  // @@protoc_insertion_point(class_scope:paraview_protobuf.ProxyState)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 0,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ProxySelectionModelState final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:paraview_protobuf.ProxySelectionModelState) */ {
 public:
  inline ProxySelectionModelState() : ProxySelectionModelState(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ProxySelectionModelState* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ProxySelectionModelState));
  }
#endif

  template <typename = void>
  explicit constexpr ProxySelectionModelState(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ProxySelectionModelState(const ProxySelectionModelState& from) : ProxySelectionModelState(nullptr, from) {}
  inline ProxySelectionModelState(ProxySelectionModelState&& from) noexcept : ProxySelectionModelState(nullptr, ::std::move(from)) {}
  inline ProxySelectionModelState& operator=(const ProxySelectionModelState& from) {
    CopyFrom(from);
    return *this;
  }
  inline ProxySelectionModelState& operator=(ProxySelectionModelState&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ProxySelectionModelState& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ProxySelectionModelState>(&ProxySelectionModelState_globals_);
  }
  static constexpr int kIndexInFileMessages = 15;
  friend void swap(ProxySelectionModelState& a, ProxySelectionModelState& b) { a.Swap(&b); }
  inline void Swap(ProxySelectionModelState* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ProxySelectionModelState* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ProxySelectionModelState* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<ProxySelectionModelState>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const ProxySelectionModelState& from) { ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from); }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const ProxySelectionModelState& from) { ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from); }

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.ProxySelectionModelState"; }

  explicit ProxySelectionModelState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ProxySelectionModelState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ProxySelectionModelState& from);
  ProxySelectionModelState(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ProxySelectionModelState&& from) noexcept
      : ProxySelectionModelState(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  inline static constexpr int kCurrentProxyFieldNumber = 52;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::PrimitiveTypeTraits< ::uint32_t >, 13, false>(current_proxy);
  inline static constexpr int kCurrentPortFieldNumber = 53;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::PrimitiveTypeTraits< ::int32_t >, 5, false>(current_port);
  inline static constexpr int kProxyFieldNumber = 50;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::RepeatedPrimitiveTypeTraits< ::uint32_t >, 13, false>(proxy);
  inline static constexpr int kPortFieldNumber = 51;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::RepeatedPrimitiveTypeTraits< ::int32_t >, 5, false>(port);
  // @@protoc_insertion_point(class_scope:paraview_protobuf.ProxySelectionModelState)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 0,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ProxyDefinitionState_ProxyXMLDefinition final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition) */ {
 public:
  inline ProxyDefinitionState_ProxyXMLDefinition() : ProxyDefinitionState_ProxyXMLDefinition(nullptr) {}
  ~ProxyDefinitionState_ProxyXMLDefinition() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ProxyDefinitionState_ProxyXMLDefinition* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ProxyDefinitionState_ProxyXMLDefinition));
  }
#endif

  template <typename = void>
  explicit constexpr ProxyDefinitionState_ProxyXMLDefinition(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ProxyDefinitionState_ProxyXMLDefinition(const ProxyDefinitionState_ProxyXMLDefinition& from) : ProxyDefinitionState_ProxyXMLDefinition(nullptr, from) {}
  inline ProxyDefinitionState_ProxyXMLDefinition(ProxyDefinitionState_ProxyXMLDefinition&& from) noexcept : ProxyDefinitionState_ProxyXMLDefinition(nullptr, ::std::move(from)) {}
  inline ProxyDefinitionState_ProxyXMLDefinition& operator=(const ProxyDefinitionState_ProxyXMLDefinition& from) {
    CopyFrom(from);
    return *this;
  }
  inline ProxyDefinitionState_ProxyXMLDefinition& operator=(ProxyDefinitionState_ProxyXMLDefinition&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ProxyDefinitionState_ProxyXMLDefinition& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ProxyDefinitionState_ProxyXMLDefinition>(&ProxyDefinitionState_ProxyXMLDefinition_globals_);
  }
  static constexpr int kIndexInFileMessages = 12;
  friend void swap(ProxyDefinitionState_ProxyXMLDefinition& a, ProxyDefinitionState_ProxyXMLDefinition& b) { a.Swap(&b); }
  inline void Swap(ProxyDefinitionState_ProxyXMLDefinition* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ProxyDefinitionState_ProxyXMLDefinition* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ProxyDefinitionState_ProxyXMLDefinition* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ProxyDefinitionState_ProxyXMLDefinition>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ProxyDefinitionState_ProxyXMLDefinition& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ProxyDefinitionState_ProxyXMLDefinition& from) { ProxyDefinitionState_ProxyXMLDefinition::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return IsInitializedImpl(*this);
  }

  private:
  static bool IsInitializedImpl(const MessageLite& msg);

  public:
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ProxyDefinitionState_ProxyXMLDefinition* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition"; }

  explicit ProxyDefinitionState_ProxyXMLDefinition(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ProxyDefinitionState_ProxyXMLDefinition(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ProxyDefinitionState_ProxyXMLDefinition& from);
  ProxyDefinitionState_ProxyXMLDefinition(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ProxyDefinitionState_ProxyXMLDefinition&& from) noexcept
      : ProxyDefinitionState_ProxyXMLDefinition(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kGroupFieldNumber = 1,
    kNameFieldNumber = 2,
    kXmlFieldNumber = 3,
  };
  // required string group = 1;
  [[nodiscard]] bool has_group()
      const;
  void clear_group() ;
  [[nodiscard]] const ::std::string& group() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_group(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_group();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_group();
  void set_allocated_group(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_group() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_group(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_group();

  public:
  // required string name = 2;
  [[nodiscard]] bool has_name()
      const;
  void clear_name() ;
  [[nodiscard]] const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // required string xml = 3;
  [[nodiscard]] bool has_xml()
      const;
  void clear_xml() ;
  [[nodiscard]] const ::std::string& xml() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_xml(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_xml();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_xml();
  void set_allocated_xml(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_xml() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_xml(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_xml();

  public:
  // @@protoc_insertion_point(class_scope:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 3,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ProxyDefinitionState_ProxyXMLDefinition& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr group_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::google::protobuf::internal::ArenaStringPtr xml_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ProxyDefinitionState final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:paraview_protobuf.ProxyDefinitionState) */ {
 public:
  inline ProxyDefinitionState() : ProxyDefinitionState(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ProxyDefinitionState* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ProxyDefinitionState));
  }
#endif

  template <typename = void>
  explicit constexpr ProxyDefinitionState(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ProxyDefinitionState(const ProxyDefinitionState& from) : ProxyDefinitionState(nullptr, from) {}
  inline ProxyDefinitionState(ProxyDefinitionState&& from) noexcept : ProxyDefinitionState(nullptr, ::std::move(from)) {}
  inline ProxyDefinitionState& operator=(const ProxyDefinitionState& from) {
    CopyFrom(from);
    return *this;
  }
  inline ProxyDefinitionState& operator=(ProxyDefinitionState&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ProxyDefinitionState& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ProxyDefinitionState>(&ProxyDefinitionState_globals_);
  }
  static constexpr int kIndexInFileMessages = 13;
  friend void swap(ProxyDefinitionState& a, ProxyDefinitionState& b) { a.Swap(&b); }
  inline void Swap(ProxyDefinitionState* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ProxyDefinitionState* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ProxyDefinitionState* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<ProxyDefinitionState>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const ProxyDefinitionState& from) { ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from); }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const ProxyDefinitionState& from) { ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from); }

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.ProxyDefinitionState"; }

  explicit ProxyDefinitionState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ProxyDefinitionState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ProxyDefinitionState& from);
  ProxyDefinitionState(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ProxyDefinitionState&& from) noexcept
      : ProxyDefinitionState(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using ProxyXMLDefinition = ProxyDefinitionState_ProxyXMLDefinition;

  // accessors -------------------------------------------------------
  inline static constexpr int kXmlDefinitionProxyFieldNumber = 35;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::RepeatedMessageTypeTraits< ::paraview_protobuf::ProxyDefinitionState_ProxyXMLDefinition >, 11, false>(xml_definition_proxy);
  inline static constexpr int kXmlCustomDefinitionProxyFieldNumber = 36;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::RepeatedMessageTypeTraits< ::paraview_protobuf::ProxyDefinitionState_ProxyXMLDefinition >, 11, false>(xml_custom_definition_proxy);
  // @@protoc_insertion_point(class_scope:paraview_protobuf.ProxyDefinitionState)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 0,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED PXMRegistrationState_Entry final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:paraview_protobuf.PXMRegistrationState.Entry) */ {
 public:
  inline PXMRegistrationState_Entry() : PXMRegistrationState_Entry(nullptr) {}
  ~PXMRegistrationState_Entry() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PXMRegistrationState_Entry* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PXMRegistrationState_Entry));
  }
#endif

  template <typename = void>
  explicit constexpr PXMRegistrationState_Entry(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline PXMRegistrationState_Entry(const PXMRegistrationState_Entry& from) : PXMRegistrationState_Entry(nullptr, from) {}
  inline PXMRegistrationState_Entry(PXMRegistrationState_Entry&& from) noexcept : PXMRegistrationState_Entry(nullptr, ::std::move(from)) {}
  inline PXMRegistrationState_Entry& operator=(const PXMRegistrationState_Entry& from) {
    CopyFrom(from);
    return *this;
  }
  inline PXMRegistrationState_Entry& operator=(PXMRegistrationState_Entry&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const PXMRegistrationState_Entry& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<PXMRegistrationState_Entry>(&PXMRegistrationState_Entry_globals_);
  }
  static constexpr int kIndexInFileMessages = 10;
  friend void swap(PXMRegistrationState_Entry& a, PXMRegistrationState_Entry& b) { a.Swap(&b); }
  inline void Swap(PXMRegistrationState_Entry* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PXMRegistrationState_Entry* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] PXMRegistrationState_Entry* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PXMRegistrationState_Entry>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PXMRegistrationState_Entry& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PXMRegistrationState_Entry& from) { PXMRegistrationState_Entry::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return IsInitializedImpl(*this);
  }

  private:
  static bool IsInitializedImpl(const MessageLite& msg);

  public:
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PXMRegistrationState_Entry* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.PXMRegistrationState.Entry"; }

  explicit PXMRegistrationState_Entry(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PXMRegistrationState_Entry(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PXMRegistrationState_Entry& from);
  PXMRegistrationState_Entry(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PXMRegistrationState_Entry&& from) noexcept
      : PXMRegistrationState_Entry(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kGroupFieldNumber = 1,
    kNameFieldNumber = 2,
    kGlobalIdFieldNumber = 3,
  };
  // optional string group = 1;
  [[nodiscard]] bool has_group()
      const;
  void clear_group() ;
  [[nodiscard]] const ::std::string& group() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_group(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_group();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_group();
  void set_allocated_group(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_group() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_group(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_group();

  public:
  // required string name = 2;
  [[nodiscard]] bool has_name()
      const;
  void clear_name() ;
  [[nodiscard]] const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // required uint64 global_id = 3;
  [[nodiscard]] bool has_global_id()
      const;
  void clear_global_id() ;
  [[nodiscard]] ::uint64_t global_id() const;
  void set_global_id(::uint64_t value);

  private:
  ::uint64_t _internal_global_id() const;
  void _internal_set_global_id(::uint64_t value);

  public:
  // @@protoc_insertion_point(class_scope:paraview_protobuf.PXMRegistrationState.Entry)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 3,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PXMRegistrationState_Entry& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr group_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::uint64_t global_id_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED PXMRegistrationState final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:paraview_protobuf.PXMRegistrationState) */ {
 public:
  inline PXMRegistrationState() : PXMRegistrationState(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PXMRegistrationState* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PXMRegistrationState));
  }
#endif

  template <typename = void>
  explicit constexpr PXMRegistrationState(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline PXMRegistrationState(const PXMRegistrationState& from) : PXMRegistrationState(nullptr, from) {}
  inline PXMRegistrationState(PXMRegistrationState&& from) noexcept : PXMRegistrationState(nullptr, ::std::move(from)) {}
  inline PXMRegistrationState& operator=(const PXMRegistrationState& from) {
    CopyFrom(from);
    return *this;
  }
  inline PXMRegistrationState& operator=(PXMRegistrationState&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const PXMRegistrationState& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<PXMRegistrationState>(&PXMRegistrationState_globals_);
  }
  static constexpr int kIndexInFileMessages = 11;
  friend void swap(PXMRegistrationState& a, PXMRegistrationState& b) { a.Swap(&b); }
  inline void Swap(PXMRegistrationState* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PXMRegistrationState* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] PXMRegistrationState* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<PXMRegistrationState>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const PXMRegistrationState& from) { ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from); }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const PXMRegistrationState& from) { ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from); }

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.PXMRegistrationState"; }

  explicit PXMRegistrationState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PXMRegistrationState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PXMRegistrationState& from);
  PXMRegistrationState(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PXMRegistrationState&& from) noexcept
      : PXMRegistrationState(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using Entry = PXMRegistrationState_Entry;

  // accessors -------------------------------------------------------
  inline static constexpr int kRegisteredProxyFieldNumber = 30;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::RepeatedMessageTypeTraits< ::paraview_protobuf::PXMRegistrationState_Entry >, 11, false>(registered_proxy);
  inline static constexpr int kRegisteredSelectionModelFieldNumber = 31;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::RepeatedMessageTypeTraits< ::paraview_protobuf::PXMRegistrationState_Entry >, 11, false>(registered_selection_model);
  inline static constexpr int kRegisteredLinkFieldNumber = 32;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::RepeatedMessageTypeTraits< ::paraview_protobuf::PXMRegistrationState_Entry >, 11, false>(registered_link);
  // @@protoc_insertion_point(class_scope:paraview_protobuf.PXMRegistrationState)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 0,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED MousePointer final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:paraview_protobuf.MousePointer) */ {
 public:
  inline MousePointer() : MousePointer(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(MousePointer* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(MousePointer));
  }
#endif

  template <typename = void>
  explicit constexpr MousePointer(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline MousePointer(const MousePointer& from) : MousePointer(nullptr, from) {}
  inline MousePointer(MousePointer&& from) noexcept : MousePointer(nullptr, ::std::move(from)) {}
  inline MousePointer& operator=(const MousePointer& from) {
    CopyFrom(from);
    return *this;
  }
  inline MousePointer& operator=(MousePointer&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const MousePointer& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<MousePointer>(&MousePointer_globals_);
  }
  static constexpr int kIndexInFileMessages = 18;
  friend void swap(MousePointer& a, MousePointer& b) { a.Swap(&b); }
  inline void Swap(MousePointer* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(MousePointer* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] MousePointer* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<MousePointer>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const MousePointer& from) { ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from); }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const MousePointer& from) { ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from); }

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.MousePointer"; }

  explicit MousePointer(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  MousePointer(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const MousePointer& from);
  MousePointer(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, MousePointer&& from) noexcept
      : MousePointer(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using Ratio = MousePointer_Ratio;
  static constexpr Ratio BOTH = MousePointer_Ratio_BOTH;
  static constexpr Ratio HEIGHT = MousePointer_Ratio_HEIGHT;
  static constexpr Ratio WIDTH = MousePointer_Ratio_WIDTH;
  [[nodiscard]] static inline bool Ratio_IsValid(int value) {
    return MousePointer_Ratio_IsValid(value);
  }
  static constexpr Ratio Ratio_MIN = MousePointer_Ratio_Ratio_MIN;
  static constexpr Ratio Ratio_MAX = MousePointer_Ratio_Ratio_MAX;
  static constexpr int Ratio_ARRAYSIZE = MousePointer_Ratio_Ratio_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL Ratio_descriptor() {
    return MousePointer_Ratio_descriptor();
  }
  template <typename T>
  [[nodiscard]] static inline const ::std::string& Ratio_Name(T value) {
    return MousePointer_Ratio_Name(value);
  }
  [[nodiscard]] static inline bool Ratio_Parse(
      ::absl::string_view name, Ratio* PROTOBUF_NONNULL value) {
    return MousePointer_Ratio_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  inline static constexpr int kViewFieldNumber = 70;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::PrimitiveTypeTraits< ::uint32_t >, 13, false>(view);
  inline static constexpr int kXFieldNumber = 71;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::PrimitiveTypeTraits< double >, 1, false>(x);
  inline static constexpr int kYFieldNumber = 72;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::PrimitiveTypeTraits< double >, 1, false>(y);
  inline static constexpr int kForceShowFieldNumber = 73;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::PrimitiveTypeTraits< bool >, 8, false>(forceShow);
  inline static constexpr int kRatioTypeFieldNumber = 74;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::EnumTypeTraits< ::paraview_protobuf::MousePointer_Ratio>, 14, false>(ratioType);
  // @@protoc_insertion_point(class_scope:paraview_protobuf.MousePointer)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 0,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Message final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:paraview_protobuf.Message) */ {
 public:
  inline Message() : Message(nullptr) {}
  ~Message() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Message* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Message));
  }
#endif

  template <typename = void>
  explicit constexpr Message(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Message(const Message& from) : Message(nullptr, from) {}
  inline Message(Message&& from) noexcept : Message(nullptr, ::std::move(from)) {}
  inline Message& operator=(const Message& from) {
    CopyFrom(from);
    return *this;
  }
  inline Message& operator=(Message&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Message& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Message>(&Message_globals_);
  }
  static constexpr int kIndexInFileMessages = 3;
  friend void swap(Message& a, Message& b) { a.Swap(&b); }
  inline void Swap(Message* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Message* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Message* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Message>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Message& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Message& from) { Message::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return IsInitializedImpl(*this);
  }

  private:
  static bool IsInitializedImpl(const MessageLite& msg);

  public:
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Message* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.Message"; }

  explicit Message(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Message(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Message& from);
  Message(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Message&& from) noexcept
      : Message(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kGlobalIdFieldNumber = 1,
    kLocationFieldNumber = 2,
    kShareOnlyFieldNumber = 3,
    kReqDefFieldNumber = 4,
    kClientIdFieldNumber = 5,
  };
  // required uint64 global_id = 1;
  [[nodiscard]] bool has_global_id()
      const;
  void clear_global_id() ;
  [[nodiscard]] ::uint64_t global_id() const;
  void set_global_id(::uint64_t value);

  private:
  ::uint64_t _internal_global_id() const;
  void _internal_set_global_id(::uint64_t value);

  public:
  // required uint32 location = 2;
  [[nodiscard]] bool has_location()
      const;
  void clear_location() ;
  [[nodiscard]] ::uint32_t location() const;
  void set_location(::uint32_t value);

  private:
  ::uint32_t _internal_location() const;
  void _internal_set_location(::uint32_t value);

  public:
  // optional bool share_only = 3 [default = false];
  [[nodiscard]] bool has_share_only()
      const;
  void clear_share_only() ;
  [[nodiscard]] bool share_only() const;
  void set_share_only(bool value);

  private:
  bool _internal_share_only() const;
  void _internal_set_share_only(bool value);

  public:
  // optional bool req_def = 4 [default = false];
  [[nodiscard]] bool has_req_def()
      const;
  void clear_req_def() ;
  [[nodiscard]] bool req_def() const;
  void set_req_def(bool value);

  private:
  bool _internal_req_def() const;
  void _internal_set_req_def(bool value);

  public:
  // optional uint32 client_id = 5 [default = 0];
  [[nodiscard]] bool has_client_id()
      const;
  void clear_client_id() ;
  [[nodiscard]] ::uint32_t client_id() const;
  void set_client_id(::uint32_t value);

  private:
  ::uint32_t _internal_client_id() const;
  void _internal_set_client_id(::uint32_t value);

  public:
  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed,
            typename = typename _proto_TypeTraits::Singular>
  PROTOBUF_FUTURE_ADD_NODISCARD inline bool HasExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Message, _proto_TypeTraits,
                                       _field_type, _is_packed>& id) const {
    return _impl_._extensions_.Has(id.number());
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline void ClearExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Message, _proto_TypeTraits,
                                       _field_type, _is_packed>& id) {
    _impl_._extensions_.ClearExtension(id.number());
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed,
            typename = typename _proto_TypeTraits::Repeated>
  PROTOBUF_FUTURE_ADD_NODISCARD inline int ExtensionSize(
      const ::google::protobuf::internal::ExtensionIdentifier<Message, _proto_TypeTraits,
                                       _field_type, _is_packed>& id) const {
    return _impl_._extensions_.ExtensionSize(id.number());
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed,
            ::std::enable_if_t<!_proto_TypeTraits::kLifetimeBound, int> = 0>
  PROTOBUF_FUTURE_ADD_NODISCARD inline
      typename _proto_TypeTraits::Singular::ConstType
      GetExtension(const ::google::protobuf::internal::ExtensionIdentifier<
                   Message, _proto_TypeTraits, _field_type, _is_packed>& id)
          const {
    return _proto_TypeTraits::Get(id.number(), _impl_._extensions_, id.default_value());
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed,
            ::std::enable_if_t<_proto_TypeTraits::kLifetimeBound, int> = 0>
  PROTOBUF_FUTURE_ADD_NODISCARD inline
      typename _proto_TypeTraits::Singular::ConstType
      GetExtension(const ::google::protobuf::internal::ExtensionIdentifier<
                   Message, _proto_TypeTraits, _field_type, _is_packed>& id)
          const ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _proto_TypeTraits::Get(GetArena(), id.number(), _impl_._extensions_,
                                  id.default_value());
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Singular::MutableType MutableExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Message, _proto_TypeTraits,
                                       _field_type, _is_packed>& id)
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _proto_TypeTraits::Mutable(GetArena(), id.number(), _field_type,
                                      &_impl_._extensions_);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline void SetExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Message, _proto_TypeTraits,
                                       _field_type, _is_packed>& id,
      typename _proto_TypeTraits::Singular::ConstType value) {
    _proto_TypeTraits::Set(GetArena(), id.number(), _field_type, value,
                           &_impl_._extensions_);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline void SetAllocatedExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Message, _proto_TypeTraits,
                                       _field_type, _is_packed>& id,
      typename _proto_TypeTraits::Singular::MutableType value) {
    _proto_TypeTraits::SetAllocated(GetArena(), id.number(), _field_type,
                                    value, &_impl_._extensions_);
  }
  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline void UnsafeArenaSetAllocatedExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Message, _proto_TypeTraits,
                                       _field_type, _is_packed>& id,
      typename _proto_TypeTraits::Singular::MutableType value) {
    _proto_TypeTraits::UnsafeArenaSetAllocated(
        GetArena(), id.number(), _field_type, value, &_impl_._extensions_);
  }
  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  [[nodiscard]] inline typename _proto_TypeTraits::Singular::MutableType
  ReleaseExtension(const ::google::protobuf::internal::ExtensionIdentifier<
                   Message, _proto_TypeTraits, _field_type, _is_packed>& id) {
    return _proto_TypeTraits::Release(GetArena(), id.number(), _field_type,
                                      &_impl_._extensions_);
  }
  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Singular::MutableType
  UnsafeArenaReleaseExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Message, _proto_TypeTraits,
                                       _field_type, _is_packed>& id) {
    return _proto_TypeTraits::UnsafeArenaRelease(
        GetArena(), id.number(), _field_type, &_impl_._extensions_);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed,
            ::std::enable_if_t<!_proto_TypeTraits::kLifetimeBound, int> = 0>
  PROTOBUF_FUTURE_ADD_NODISCARD inline
      typename _proto_TypeTraits::Repeated::ConstType
      GetExtension(
          const ::google::protobuf::internal::ExtensionIdentifier<Message, _proto_TypeTraits,
                                           _field_type, _is_packed>& id,
          int index) const {
    return _proto_TypeTraits::Get(id.number(), _impl_._extensions_, index);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed,
            ::std::enable_if_t<_proto_TypeTraits::kLifetimeBound, int> = 0>
  PROTOBUF_FUTURE_ADD_NODISCARD inline
      typename _proto_TypeTraits::Repeated::ConstType
      GetExtension(
          const ::google::protobuf::internal::ExtensionIdentifier<Message, _proto_TypeTraits,
                                           _field_type, _is_packed>& id,
          int index) const ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _proto_TypeTraits::Get(id.number(), _impl_._extensions_, index);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Repeated::MutableType MutableExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Message, _proto_TypeTraits,
                                       _field_type, _is_packed>& id,
      int index) ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _proto_TypeTraits::Mutable(id.number(), index, &_impl_._extensions_);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline void SetExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Message, _proto_TypeTraits,
                                       _field_type, _is_packed>& id,
      int index, typename _proto_TypeTraits::Repeated::ConstType value) {
    _proto_TypeTraits::Set(id.number(), index, value, &_impl_._extensions_);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Repeated::MutableType AddExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Message, _proto_TypeTraits,
                                       _field_type, _is_packed>& id)
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    typename _proto_TypeTraits::Repeated::MutableType to_add =
        _proto_TypeTraits::Add(GetArena(), id.number(), _field_type,
                               &_impl_._extensions_);
    return to_add;
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline void AddExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Message, _proto_TypeTraits,
                                       _field_type, _is_packed>& id,
      typename _proto_TypeTraits::Repeated::ConstType value) {
    _proto_TypeTraits::Add(GetArena(), id.number(), _field_type, _is_packed,
                           value, &_impl_._extensions_);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  PROTOBUF_FUTURE_ADD_NODISCARD inline const typename _proto_TypeTraits::
      Repeated::RepeatedFieldType&
      GetRepeatedExtension(
          const ::google::protobuf::internal::ExtensionIdentifier<Message, _proto_TypeTraits,
                                           _field_type, _is_packed>& id)
          const ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _proto_TypeTraits::GetRepeated(id.number(), _impl_._extensions_);
  }

  template <typename _proto_TypeTraits, ::google::protobuf::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Repeated::RepeatedFieldType* PROTOBUF_NONNULL
  MutableRepeatedExtension(
      const ::google::protobuf::internal::ExtensionIdentifier<Message, _proto_TypeTraits,
                                       _field_type, _is_packed>& id)
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _proto_TypeTraits::MutableRepeated(
        GetArena(), id.number(), _field_type, _is_packed, &_impl_._extensions_);
  }
  // @@protoc_insertion_point(class_scope:paraview_protobuf.Message)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 5,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Message& from_msg);
    ::google::protobuf::internal::ExtensionSet _extensions_;
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::uint64_t global_id_;
    ::uint32_t location_;
    bool share_only_;
    bool req_def_;
    ::uint32_t client_id_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED LinkState_LinkDescription final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:paraview_protobuf.LinkState.LinkDescription) */ {
 public:
  inline LinkState_LinkDescription() : LinkState_LinkDescription(nullptr) {}
  ~LinkState_LinkDescription() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(LinkState_LinkDescription* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(LinkState_LinkDescription));
  }
#endif

  template <typename = void>
  explicit constexpr LinkState_LinkDescription(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline LinkState_LinkDescription(const LinkState_LinkDescription& from) : LinkState_LinkDescription(nullptr, from) {}
  inline LinkState_LinkDescription(LinkState_LinkDescription&& from) noexcept : LinkState_LinkDescription(nullptr, ::std::move(from)) {}
  inline LinkState_LinkDescription& operator=(const LinkState_LinkDescription& from) {
    CopyFrom(from);
    return *this;
  }
  inline LinkState_LinkDescription& operator=(LinkState_LinkDescription&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const LinkState_LinkDescription& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<LinkState_LinkDescription>(&LinkState_LinkDescription_globals_);
  }
  static constexpr int kIndexInFileMessages = 16;
  friend void swap(LinkState_LinkDescription& a, LinkState_LinkDescription& b) { a.Swap(&b); }
  inline void Swap(LinkState_LinkDescription* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(LinkState_LinkDescription* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] LinkState_LinkDescription* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<LinkState_LinkDescription>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const LinkState_LinkDescription& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const LinkState_LinkDescription& from) { LinkState_LinkDescription::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return IsInitializedImpl(*this);
  }

  private:
  static bool IsInitializedImpl(const MessageLite& msg);

  public:
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(LinkState_LinkDescription* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.LinkState.LinkDescription"; }

  explicit LinkState_LinkDescription(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  LinkState_LinkDescription(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const LinkState_LinkDescription& from);
  LinkState_LinkDescription(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, LinkState_LinkDescription&& from) noexcept
      : LinkState_LinkDescription(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using Direction = LinkState_LinkDescription_Direction;
  static constexpr Direction NONE = LinkState_LinkDescription_Direction_NONE;
  static constexpr Direction INPUT = LinkState_LinkDescription_Direction_INPUT;
  static constexpr Direction OUTPUT = LinkState_LinkDescription_Direction_OUTPUT;
  [[nodiscard]] static inline bool Direction_IsValid(int value) {
    return LinkState_LinkDescription_Direction_IsValid(value);
  }
  static constexpr Direction Direction_MIN = LinkState_LinkDescription_Direction_Direction_MIN;
  static constexpr Direction Direction_MAX = LinkState_LinkDescription_Direction_Direction_MAX;
  static constexpr int Direction_ARRAYSIZE = LinkState_LinkDescription_Direction_Direction_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL Direction_descriptor() {
    return LinkState_LinkDescription_Direction_descriptor();
  }
  template <typename T>
  [[nodiscard]] static inline const ::std::string& Direction_Name(T value) {
    return LinkState_LinkDescription_Direction_Name(value);
  }
  [[nodiscard]] static inline bool Direction_Parse(
      ::absl::string_view name, Direction* PROTOBUF_NONNULL value) {
    return LinkState_LinkDescription_Direction_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  enum : int {
    kPropertyNameFieldNumber = 2,
    kProxyFieldNumber = 1,
    kDirectionFieldNumber = 3,
  };
  // optional string property_name = 2;
  [[nodiscard]] bool has_property_name()
      const;
  void clear_property_name() ;
  [[nodiscard]] const ::std::string& property_name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_property_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_property_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_property_name();
  void set_allocated_property_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_property_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_property_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_property_name();

  public:
  // required uint32 proxy = 1;
  [[nodiscard]] bool has_proxy()
      const;
  void clear_proxy() ;
  [[nodiscard]] ::uint32_t proxy() const;
  void set_proxy(::uint32_t value);

  private:
  ::uint32_t _internal_proxy() const;
  void _internal_set_proxy(::uint32_t value);

  public:
  // required .paraview_protobuf.LinkState.LinkDescription.Direction direction = 3;
  [[nodiscard]] bool has_direction()
      const;
  void clear_direction() ;
  [[nodiscard]] ::paraview_protobuf::LinkState_LinkDescription_Direction direction() const;
  void set_direction(::paraview_protobuf::LinkState_LinkDescription_Direction value);

  private:
  ::paraview_protobuf::LinkState_LinkDescription_Direction _internal_direction() const;
  void _internal_set_direction(::paraview_protobuf::LinkState_LinkDescription_Direction value);

  public:
  // @@protoc_insertion_point(class_scope:paraview_protobuf.LinkState.LinkDescription)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 3,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const LinkState_LinkDescription& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr property_name_;
    ::uint32_t proxy_;
    int direction_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED LinkState final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:paraview_protobuf.LinkState) */ {
 public:
  inline LinkState() : LinkState(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(LinkState* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(LinkState));
  }
#endif

  template <typename = void>
  explicit constexpr LinkState(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline LinkState(const LinkState& from) : LinkState(nullptr, from) {}
  inline LinkState(LinkState&& from) noexcept : LinkState(nullptr, ::std::move(from)) {}
  inline LinkState& operator=(const LinkState& from) {
    CopyFrom(from);
    return *this;
  }
  inline LinkState& operator=(LinkState&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const LinkState& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<LinkState>(&LinkState_globals_);
  }
  static constexpr int kIndexInFileMessages = 17;
  friend void swap(LinkState& a, LinkState& b) { a.Swap(&b); }
  inline void Swap(LinkState* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(LinkState* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] LinkState* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<LinkState>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const LinkState& from) { ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from); }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const LinkState& from) { ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from); }

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.LinkState"; }

  explicit LinkState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  LinkState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const LinkState& from);
  LinkState(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, LinkState&& from) noexcept
      : LinkState(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using LinkDescription = LinkState_LinkDescription;

  // accessors -------------------------------------------------------
  inline static constexpr int kPropagateUpdateFieldNumber = 60;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::PrimitiveTypeTraits< bool >, 8, false>(propagate_update);
  inline static constexpr int kEnabledFieldNumber = 61;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::PrimitiveTypeTraits< bool >, 8, false>(enabled);
  inline static constexpr int kSyncInteractiveRendersFieldNumber = 62;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::PrimitiveTypeTraits< bool >, 8, false>(sync_interactive_renders);
  inline static constexpr int kLinkFieldNumber = 63;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::RepeatedMessageTypeTraits< ::paraview_protobuf::LinkState_LinkDescription >, 11, false>(link);
  inline static constexpr int kExceptionPropertyFieldNumber = 64;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::RepeatedStringTypeTraits, 12, false>(exception_property);
  // @@protoc_insertion_point(class_scope:paraview_protobuf.LinkState)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 0,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED DefinitionHeader final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:paraview_protobuf.DefinitionHeader) */ {
 public:
  inline DefinitionHeader() : DefinitionHeader(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(DefinitionHeader* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(DefinitionHeader));
  }
#endif

  template <typename = void>
  explicit constexpr DefinitionHeader(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline DefinitionHeader(const DefinitionHeader& from) : DefinitionHeader(nullptr, from) {}
  inline DefinitionHeader(DefinitionHeader&& from) noexcept : DefinitionHeader(nullptr, ::std::move(from)) {}
  inline DefinitionHeader& operator=(const DefinitionHeader& from) {
    CopyFrom(from);
    return *this;
  }
  inline DefinitionHeader& operator=(DefinitionHeader&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const DefinitionHeader& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<DefinitionHeader>(&DefinitionHeader_globals_);
  }
  static constexpr int kIndexInFileMessages = 4;
  friend void swap(DefinitionHeader& a, DefinitionHeader& b) { a.Swap(&b); }
  inline void Swap(DefinitionHeader* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(DefinitionHeader* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] DefinitionHeader* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<DefinitionHeader>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const DefinitionHeader& from) { ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from); }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const DefinitionHeader& from) { ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from); }

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.DefinitionHeader"; }

  explicit DefinitionHeader(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  DefinitionHeader(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const DefinitionHeader& from);
  DefinitionHeader(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, DefinitionHeader&& from) noexcept
      : DefinitionHeader(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  inline static constexpr int kClientClassFieldNumber = 10;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::StringTypeTraits, 12, false>(client_class);
  inline static constexpr int kServerClassFieldNumber = 11;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::StringTypeTraits, 12, false>(server_class);
  // @@protoc_insertion_point(class_scope:paraview_protobuf.DefinitionHeader)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 0,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ClientsInformation_ClientInfo final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:paraview_protobuf.ClientsInformation.ClientInfo) */ {
 public:
  inline ClientsInformation_ClientInfo() : ClientsInformation_ClientInfo(nullptr) {}
  ~ClientsInformation_ClientInfo() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ClientsInformation_ClientInfo* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ClientsInformation_ClientInfo));
  }
#endif

  template <typename = void>
  explicit constexpr ClientsInformation_ClientInfo(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ClientsInformation_ClientInfo(const ClientsInformation_ClientInfo& from) : ClientsInformation_ClientInfo(nullptr, from) {}
  inline ClientsInformation_ClientInfo(ClientsInformation_ClientInfo&& from) noexcept : ClientsInformation_ClientInfo(nullptr, ::std::move(from)) {}
  inline ClientsInformation_ClientInfo& operator=(const ClientsInformation_ClientInfo& from) {
    CopyFrom(from);
    return *this;
  }
  inline ClientsInformation_ClientInfo& operator=(ClientsInformation_ClientInfo&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ClientsInformation_ClientInfo& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ClientsInformation_ClientInfo>(&ClientsInformation_ClientInfo_globals_);
  }
  static constexpr int kIndexInFileMessages = 20;
  friend void swap(ClientsInformation_ClientInfo& a, ClientsInformation_ClientInfo& b) { a.Swap(&b); }
  inline void Swap(ClientsInformation_ClientInfo* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ClientsInformation_ClientInfo* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ClientsInformation_ClientInfo* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ClientsInformation_ClientInfo>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ClientsInformation_ClientInfo& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ClientsInformation_ClientInfo& from) { ClientsInformation_ClientInfo::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return IsInitializedImpl(*this);
  }

  private:
  static bool IsInitializedImpl(const MessageLite& msg);

  public:
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ClientsInformation_ClientInfo* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.ClientsInformation.ClientInfo"; }

  explicit ClientsInformation_ClientInfo(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ClientsInformation_ClientInfo(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ClientsInformation_ClientInfo& from);
  ClientsInformation_ClientInfo(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ClientsInformation_ClientInfo&& from) noexcept
      : ClientsInformation_ClientInfo(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kNameFieldNumber = 2,
    kUserFieldNumber = 1,
    kIsMasterFieldNumber = 3,
    kFollowCamFieldNumber = 4,
    kDisableFurtherConnectionsFieldNumber = 5,
    kConnectIdFieldNumber = 6,
  };
  // optional string name = 2;
  [[nodiscard]] bool has_name()
      const;
  void clear_name() ;
  [[nodiscard]] const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // required uint32 user = 1;
  [[nodiscard]] bool has_user()
      const;
  void clear_user() ;
  [[nodiscard]] ::uint32_t user() const;
  void set_user(::uint32_t value);

  private:
  ::uint32_t _internal_user() const;
  void _internal_set_user(::uint32_t value);

  public:
  // optional bool is_master = 3 [default = false];
  [[nodiscard]] bool has_is_master()
      const;
  void clear_is_master() ;
  [[nodiscard]] bool is_master() const;
  void set_is_master(bool value);

  private:
  bool _internal_is_master() const;
  void _internal_set_is_master(bool value);

  public:
  // optional bool follow_cam = 4 [default = false];
  [[nodiscard]] bool has_follow_cam()
      const;
  void clear_follow_cam() ;
  [[nodiscard]] bool follow_cam() const;
  void set_follow_cam(bool value);

  private:
  bool _internal_follow_cam() const;
  void _internal_set_follow_cam(bool value);

  public:
  // optional bool disable_further_connections = 5 [default = false];
  [[nodiscard]] bool has_disable_further_connections()
      const;
  void clear_disable_further_connections() ;
  [[nodiscard]] bool disable_further_connections() const;
  void set_disable_further_connections(bool value);

  private:
  bool _internal_disable_further_connections() const;
  void _internal_set_disable_further_connections(bool value);

  public:
  // optional uint32 connect_id = 6 [default = 0];
  [[nodiscard]] bool has_connect_id()
      const;
  void clear_connect_id() ;
  [[nodiscard]] ::uint32_t connect_id() const;
  void set_connect_id(::uint32_t value);

  private:
  ::uint32_t _internal_connect_id() const;
  void _internal_set_connect_id(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:paraview_protobuf.ClientsInformation.ClientInfo)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 6,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ClientsInformation_ClientInfo& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::uint32_t user_;
    bool is_master_;
    bool follow_cam_;
    bool disable_further_connections_;
    ::uint32_t connect_id_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ClientsInformation final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:paraview_protobuf.ClientsInformation) */ {
 public:
  inline ClientsInformation() : ClientsInformation(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ClientsInformation* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ClientsInformation));
  }
#endif

  template <typename = void>
  explicit constexpr ClientsInformation(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ClientsInformation(const ClientsInformation& from) : ClientsInformation(nullptr, from) {}
  inline ClientsInformation(ClientsInformation&& from) noexcept : ClientsInformation(nullptr, ::std::move(from)) {}
  inline ClientsInformation& operator=(const ClientsInformation& from) {
    CopyFrom(from);
    return *this;
  }
  inline ClientsInformation& operator=(ClientsInformation&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ClientsInformation& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ClientsInformation>(&ClientsInformation_globals_);
  }
  static constexpr int kIndexInFileMessages = 21;
  friend void swap(ClientsInformation& a, ClientsInformation& b) { a.Swap(&b); }
  inline void Swap(ClientsInformation* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ClientsInformation* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ClientsInformation* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<ClientsInformation>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const ClientsInformation& from) { ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from); }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const ClientsInformation& from) { ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from); }

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.ClientsInformation"; }

  explicit ClientsInformation(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ClientsInformation(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ClientsInformation& from);
  ClientsInformation(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ClientsInformation&& from) noexcept
      : ClientsInformation(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using ClientInfo = ClientsInformation_ClientInfo;

  // accessors -------------------------------------------------------
  inline static constexpr int kUserFieldNumber = 80;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::RepeatedMessageTypeTraits< ::paraview_protobuf::ClientsInformation_ClientInfo >, 11, false>(user);
  // @@protoc_insertion_point(class_scope:paraview_protobuf.ClientsInformation)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 0,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ChatMessage final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:paraview_protobuf.ChatMessage) */ {
 public:
  inline ChatMessage() : ChatMessage(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ChatMessage* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ChatMessage));
  }
#endif

  template <typename = void>
  explicit constexpr ChatMessage(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ChatMessage(const ChatMessage& from) : ChatMessage(nullptr, from) {}
  inline ChatMessage(ChatMessage&& from) noexcept : ChatMessage(nullptr, ::std::move(from)) {}
  inline ChatMessage& operator=(const ChatMessage& from) {
    CopyFrom(from);
    return *this;
  }
  inline ChatMessage& operator=(ChatMessage&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ChatMessage& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ChatMessage>(&ChatMessage_globals_);
  }
  static constexpr int kIndexInFileMessages = 22;
  friend void swap(ChatMessage& a, ChatMessage& b) { a.Swap(&b); }
  inline void Swap(ChatMessage* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ChatMessage* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ChatMessage* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<ChatMessage>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const ChatMessage& from) { ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from); }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const ChatMessage& from) { ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from); }

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.ChatMessage"; }

  explicit ChatMessage(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ChatMessage(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ChatMessage& from);
  ChatMessage(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ChatMessage&& from) noexcept
      : ChatMessage(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  inline static constexpr int kAuthorFieldNumber = 84;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::PrimitiveTypeTraits< ::uint32_t >, 13, false>(author);
  inline static constexpr int kTxtFieldNumber = 85;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::StringTypeTraits, 12, false>(txt);
  // @@protoc_insertion_point(class_scope:paraview_protobuf.ChatMessage)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 0,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ChartViewBounds final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:paraview_protobuf.ChartViewBounds) */ {
 public:
  inline ChartViewBounds() : ChartViewBounds(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ChartViewBounds* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ChartViewBounds));
  }
#endif

  template <typename = void>
  explicit constexpr ChartViewBounds(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ChartViewBounds(const ChartViewBounds& from) : ChartViewBounds(nullptr, from) {}
  inline ChartViewBounds(ChartViewBounds&& from) noexcept : ChartViewBounds(nullptr, ::std::move(from)) {}
  inline ChartViewBounds& operator=(const ChartViewBounds& from) {
    CopyFrom(from);
    return *this;
  }
  inline ChartViewBounds& operator=(ChartViewBounds&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ChartViewBounds& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ChartViewBounds>(&ChartViewBounds_globals_);
  }
  static constexpr int kIndexInFileMessages = 19;
  friend void swap(ChartViewBounds& a, ChartViewBounds& b) { a.Swap(&b); }
  inline void Swap(ChartViewBounds* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ChartViewBounds* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ChartViewBounds* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<ChartViewBounds>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const ChartViewBounds& from) { ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from); }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const ChartViewBounds& from) { ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from); }

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.ChartViewBounds"; }

  explicit ChartViewBounds(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ChartViewBounds(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ChartViewBounds& from);
  ChartViewBounds(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ChartViewBounds&& from) noexcept
      : ChartViewBounds(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  inline static constexpr int kViewFieldNumber = 75;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::PrimitiveTypeTraits< ::uint32_t >, 13, false>(view);
  inline static constexpr int kRangeFieldNumber = 76;
  static ::google::protobuf::internal::ExtensionIdentifier<
      ::paraview_protobuf::Message, ::google::protobuf::internal::RepeatedPrimitiveTypeTraits< double >, 1, false>(range);
  // @@protoc_insertion_point(class_scope:paraview_protobuf.ChartViewBounds)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 0,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED VariantList final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:paraview_protobuf.VariantList) */ {
 public:
  inline VariantList() : VariantList(nullptr) {}
  ~VariantList() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(VariantList* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(VariantList));
  }
#endif

  template <typename = void>
  explicit constexpr VariantList(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline VariantList(const VariantList& from) : VariantList(nullptr, from) {}
  inline VariantList(VariantList&& from) noexcept : VariantList(nullptr, ::std::move(from)) {}
  inline VariantList& operator=(const VariantList& from) {
    CopyFrom(from);
    return *this;
  }
  inline VariantList& operator=(VariantList&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const VariantList& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<VariantList>(&VariantList_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(VariantList& a, VariantList& b) { a.Swap(&b); }
  inline void Swap(VariantList* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(VariantList* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] VariantList* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<VariantList>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const VariantList& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const VariantList& from) { VariantList::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return IsInitializedImpl(*this);
  }

  private:
  static bool IsInitializedImpl(const MessageLite& msg);

  public:
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(VariantList* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.VariantList"; }

  explicit VariantList(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  VariantList(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const VariantList& from);
  VariantList(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, VariantList&& from) noexcept
      : VariantList(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kVariantFieldNumber = 1,
  };
  // repeated .paraview_protobuf.Variant variant = 1;
  [[nodiscard]] int variant_size()
      const;
  private:
  int _internal_variant_size() const;

  public:
  void clear_variant() ;
  [[nodiscard]] const ::paraview_protobuf::Variant& variant(int index) const;
  [[nodiscard]] ::paraview_protobuf::Variant* PROTOBUF_NONNULL mutable_variant(int index);
  ::paraview_protobuf::Variant* PROTOBUF_NONNULL add_variant();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Variant>&
  variant() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Variant>* PROTOBUF_NONNULL
  mutable_variant();

  private:
  const ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Variant>& _internal_variant() const;
  ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Variant>* PROTOBUF_NONNULL _internal_mutable_variant();

  public:
  // @@protoc_insertion_point(class_scope:paraview_protobuf.VariantList)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const VariantList& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::paraview_protobuf::Variant > variant_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ProxyState_UserData final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:paraview_protobuf.ProxyState.UserData) */ {
 public:
  inline ProxyState_UserData() : ProxyState_UserData(nullptr) {}
  ~ProxyState_UserData() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ProxyState_UserData* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ProxyState_UserData));
  }
#endif

  template <typename = void>
  explicit constexpr ProxyState_UserData(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ProxyState_UserData(const ProxyState_UserData& from) : ProxyState_UserData(nullptr, from) {}
  inline ProxyState_UserData(ProxyState_UserData&& from) noexcept : ProxyState_UserData(nullptr, ::std::move(from)) {}
  inline ProxyState_UserData& operator=(const ProxyState_UserData& from) {
    CopyFrom(from);
    return *this;
  }
  inline ProxyState_UserData& operator=(ProxyState_UserData&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ProxyState_UserData& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ProxyState_UserData>(&ProxyState_UserData_globals_);
  }
  static constexpr int kIndexInFileMessages = 7;
  friend void swap(ProxyState_UserData& a, ProxyState_UserData& b) { a.Swap(&b); }
  inline void Swap(ProxyState_UserData* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ProxyState_UserData* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ProxyState_UserData* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ProxyState_UserData>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ProxyState_UserData& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ProxyState_UserData& from) { ProxyState_UserData::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return IsInitializedImpl(*this);
  }

  private:
  static bool IsInitializedImpl(const MessageLite& msg);

  public:
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ProxyState_UserData* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.ProxyState.UserData"; }

  explicit ProxyState_UserData(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ProxyState_UserData(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ProxyState_UserData& from);
  ProxyState_UserData(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ProxyState_UserData&& from) noexcept
      : ProxyState_UserData(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kVariantFieldNumber = 2,
    kKeyFieldNumber = 1,
  };
  // repeated .paraview_protobuf.Variant variant = 2;
  [[nodiscard]] int variant_size()
      const;
  private:
  int _internal_variant_size() const;

  public:
  void clear_variant() ;
  [[nodiscard]] const ::paraview_protobuf::Variant& variant(int index) const;
  [[nodiscard]] ::paraview_protobuf::Variant* PROTOBUF_NONNULL mutable_variant(int index);
  ::paraview_protobuf::Variant* PROTOBUF_NONNULL add_variant();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Variant>&
  variant() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Variant>* PROTOBUF_NONNULL
  mutable_variant();

  private:
  const ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Variant>& _internal_variant() const;
  ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Variant>* PROTOBUF_NONNULL _internal_mutable_variant();

  public:
  // required string key = 1;
  [[nodiscard]] bool has_key()
      const;
  void clear_key() ;
  [[nodiscard]] const ::std::string& key() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_key(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_key();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_key();
  void set_allocated_key(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_key() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_key(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_key();

  public:
  // @@protoc_insertion_point(class_scope:paraview_protobuf.ProxyState.UserData)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ProxyState_UserData& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::paraview_protobuf::Variant > variant_;
    ::google::protobuf::internal::ArenaStringPtr key_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED MessageCollection final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:paraview_protobuf.MessageCollection) */ {
 public:
  inline MessageCollection() : MessageCollection(nullptr) {}
  ~MessageCollection() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(MessageCollection* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(MessageCollection));
  }
#endif

  template <typename = void>
  explicit constexpr MessageCollection(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline MessageCollection(const MessageCollection& from) : MessageCollection(nullptr, from) {}
  inline MessageCollection(MessageCollection&& from) noexcept : MessageCollection(nullptr, ::std::move(from)) {}
  inline MessageCollection& operator=(const MessageCollection& from) {
    CopyFrom(from);
    return *this;
  }
  inline MessageCollection& operator=(MessageCollection&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const MessageCollection& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<MessageCollection>(&MessageCollection_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(MessageCollection& a, MessageCollection& b) { a.Swap(&b); }
  inline void Swap(MessageCollection* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(MessageCollection* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] MessageCollection* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<MessageCollection>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const MessageCollection& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const MessageCollection& from) { MessageCollection::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return IsInitializedImpl(*this);
  }

  private:
  static bool IsInitializedImpl(const MessageLite& msg);

  public:
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(MessageCollection* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.MessageCollection"; }

  explicit MessageCollection(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  MessageCollection(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const MessageCollection& from);
  MessageCollection(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, MessageCollection&& from) noexcept
      : MessageCollection(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kItemFieldNumber = 1,
  };
  // repeated .paraview_protobuf.Message item = 1;
  [[nodiscard]] int item_size()
      const;
  private:
  int _internal_item_size() const;

  public:
  void clear_item() ;
  [[nodiscard]] const ::paraview_protobuf::Message& item(int index) const;
  [[nodiscard]] ::paraview_protobuf::Message* PROTOBUF_NONNULL mutable_item(int index);
  ::paraview_protobuf::Message* PROTOBUF_NONNULL add_item();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Message>&
  item() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Message>* PROTOBUF_NONNULL
  mutable_item();

  private:
  const ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Message>& _internal_item() const;
  ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Message>* PROTOBUF_NONNULL _internal_mutable_item();

  public:
  // @@protoc_insertion_point(class_scope:paraview_protobuf.MessageCollection)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const MessageCollection& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::paraview_protobuf::Message > item_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};
// -------------------------------------------------------------------

class VTKREMOTINGSERVERMANAGER_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ProxyState_Property final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:paraview_protobuf.ProxyState.Property) */ {
 public:
  inline ProxyState_Property() : ProxyState_Property(nullptr) {}
  ~ProxyState_Property() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ProxyState_Property* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ProxyState_Property));
  }
#endif

  template <typename = void>
  explicit constexpr ProxyState_Property(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ProxyState_Property(const ProxyState_Property& from) : ProxyState_Property(nullptr, from) {}
  inline ProxyState_Property(ProxyState_Property&& from) noexcept : ProxyState_Property(nullptr, ::std::move(from)) {}
  inline ProxyState_Property& operator=(const ProxyState_Property& from) {
    CopyFrom(from);
    return *this;
  }
  inline ProxyState_Property& operator=(ProxyState_Property&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ProxyState_Property& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ProxyState_Property>(&ProxyState_Property_globals_);
  }
  static constexpr int kIndexInFileMessages = 8;
  friend void swap(ProxyState_Property& a, ProxyState_Property& b) { a.Swap(&b); }
  inline void Swap(ProxyState_Property* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ProxyState_Property* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ProxyState_Property* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ProxyState_Property>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ProxyState_Property& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ProxyState_Property& from) { ProxyState_Property::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return IsInitializedImpl(*this);
  }

  private:
  static bool IsInitializedImpl(const MessageLite& msg);

  public:
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ProxyState_Property* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "paraview_protobuf.ProxyState.Property"; }

  explicit ProxyState_Property(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ProxyState_Property(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ProxyState_Property& from);
  ProxyState_Property(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ProxyState_Property&& from) noexcept
      : ProxyState_Property(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kUserDataFieldNumber = 3,
    kNameFieldNumber = 1,
    kValueFieldNumber = 2,
  };
  // repeated .paraview_protobuf.ProxyState.UserData user_data = 3;
  [[nodiscard]] int user_data_size()
      const;
  private:
  int _internal_user_data_size() const;

  public:
  void clear_user_data() ;
  [[nodiscard]] const ::paraview_protobuf::ProxyState_UserData& user_data(int index) const;
  [[nodiscard]] ::paraview_protobuf::ProxyState_UserData* PROTOBUF_NONNULL mutable_user_data(int index);
  ::paraview_protobuf::ProxyState_UserData* PROTOBUF_NONNULL add_user_data();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::paraview_protobuf::ProxyState_UserData>&
  user_data() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::paraview_protobuf::ProxyState_UserData>* PROTOBUF_NONNULL
  mutable_user_data();

  private:
  const ::google::protobuf::RepeatedPtrField<::paraview_protobuf::ProxyState_UserData>& _internal_user_data() const;
  ::google::protobuf::RepeatedPtrField<::paraview_protobuf::ProxyState_UserData>* PROTOBUF_NONNULL _internal_mutable_user_data();

  public:
  // required string name = 1;
  [[nodiscard]] bool has_name()
      const;
  void clear_name() ;
  [[nodiscard]] const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // optional .paraview_protobuf.Variant value = 2;
  [[nodiscard]] bool has_value()
      const;
  void clear_value() ;
  [[nodiscard]] const ::paraview_protobuf::Variant& value() const;
  [[nodiscard]] ::paraview_protobuf::Variant* PROTOBUF_NULLABLE release_value();
  ::paraview_protobuf::Variant* PROTOBUF_NONNULL mutable_value();
  void set_allocated_value(::paraview_protobuf::Variant* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_value(::paraview_protobuf::Variant* PROTOBUF_NULLABLE value);
  ::paraview_protobuf::Variant* PROTOBUF_NULLABLE unsafe_arena_release_value();

  private:
  const ::paraview_protobuf::Variant& _internal_value() const;
  ::paraview_protobuf::Variant* PROTOBUF_NONNULL _internal_mutable_value();

  public:
  // @@protoc_insertion_point(class_scope:paraview_protobuf.ProxyState.Property)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 3,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ProxyState_Property& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::paraview_protobuf::ProxyState_UserData > user_data_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::paraview_protobuf::Variant* PROTOBUF_NULLABLE value_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_vtkPVMessage_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// Variant

// required .paraview_protobuf.Variant.Type type = 1;
inline bool Variant::has_type() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000080U);
  return value;
}
inline void Variant::clear_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline ::paraview_protobuf::Variant_Type Variant::type() const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.Variant.type)
  return _internal_type();
}
inline void Variant::set_type(::paraview_protobuf::Variant_Type value) {
  _internal_set_type(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:paraview_protobuf.Variant.type)
}
inline ::paraview_protobuf::Variant_Type Variant::_internal_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::paraview_protobuf::Variant_Type>(_impl_.type_);
}
inline void Variant::_internal_set_type(::paraview_protobuf::Variant_Type value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);

                                          assert(::google::protobuf::internal::ValidateEnum(
                                              value, ::paraview_protobuf::Variant_Type_internal_data_));
                                          _impl_.type_ = value;
}

// repeated sint64 idtype = 3;
inline int Variant::_internal_idtype_size() const {
  return _internal_idtype().size();
}
inline int Variant::idtype_size() const {
  return _internal_idtype_size();
}
inline void Variant::clear_idtype() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.idtype_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::int64_t Variant::idtype(int index) const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.Variant.idtype)
  return _internal_idtype().Get(index);
}
inline void Variant::set_idtype(int index, ::int64_t value) {
  _internal_mutable_idtype()->Set(index, value);
  // @@protoc_insertion_point(field_set:paraview_protobuf.Variant.idtype)
}
inline void Variant::add_idtype(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_idtype()
      ->InternalAddWithArena<const ::google::protobuf::MessageLite*>(
          internal_visibility(), this, value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:paraview_protobuf.Variant.idtype)
}
inline const ::google::protobuf::RepeatedField<::int64_t>& Variant::idtype() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:paraview_protobuf.Variant.idtype)
  return _internal_idtype();
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL Variant::mutable_idtype()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:paraview_protobuf.Variant.idtype)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_idtype();
}
inline const ::google::protobuf::RepeatedField<::int64_t>&
Variant::_internal_idtype() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.idtype_;
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL
Variant::_internal_mutable_idtype() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.idtype_;
}

// repeated sint32 integer = 4;
inline int Variant::_internal_integer_size() const {
  return _internal_integer().size();
}
inline int Variant::integer_size() const {
  return _internal_integer_size();
}
inline void Variant::clear_integer() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.integer_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::int32_t Variant::integer(int index) const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.Variant.integer)
  return _internal_integer().Get(index);
}
inline void Variant::set_integer(int index, ::int32_t value) {
  _internal_mutable_integer()->Set(index, value);
  // @@protoc_insertion_point(field_set:paraview_protobuf.Variant.integer)
}
inline void Variant::add_integer(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_integer()
      ->InternalAddWithArena<const ::google::protobuf::MessageLite*>(
          internal_visibility(), this, value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:paraview_protobuf.Variant.integer)
}
inline const ::google::protobuf::RepeatedField<::int32_t>& Variant::integer() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:paraview_protobuf.Variant.integer)
  return _internal_integer();
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL Variant::mutable_integer()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:paraview_protobuf.Variant.integer)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_integer();
}
inline const ::google::protobuf::RepeatedField<::int32_t>&
Variant::_internal_integer() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.integer_;
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL
Variant::_internal_mutable_integer() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.integer_;
}

// repeated double float64 = 5;
inline int Variant::_internal_float64_size() const {
  return _internal_float64().size();
}
inline int Variant::float64_size() const {
  return _internal_float64_size();
}
inline void Variant::clear_float64() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.float64_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline double Variant::float64(int index) const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.Variant.float64)
  return _internal_float64().Get(index);
}
inline void Variant::set_float64(int index, double value) {
  _internal_mutable_float64()->Set(index, value);
  // @@protoc_insertion_point(field_set:paraview_protobuf.Variant.float64)
}
inline void Variant::add_float64(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_float64()
      ->InternalAddWithArena<const ::google::protobuf::MessageLite*>(
          internal_visibility(), this, value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_add:paraview_protobuf.Variant.float64)
}
inline const ::google::protobuf::RepeatedField<double>& Variant::float64() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:paraview_protobuf.Variant.float64)
  return _internal_float64();
}
inline ::google::protobuf::RepeatedField<double>* PROTOBUF_NONNULL Variant::mutable_float64()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_mutable_list:paraview_protobuf.Variant.float64)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_float64();
}
inline const ::google::protobuf::RepeatedField<double>&
Variant::_internal_float64() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.float64_;
}
inline ::google::protobuf::RepeatedField<double>* PROTOBUF_NONNULL
Variant::_internal_mutable_float64() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.float64_;
}

// repeated uint64 proxy_global_id = 6;
inline int Variant::_internal_proxy_global_id_size() const {
  return _internal_proxy_global_id().size();
}
inline int Variant::proxy_global_id_size() const {
  return _internal_proxy_global_id_size();
}
inline void Variant::clear_proxy_global_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.proxy_global_id_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t Variant::proxy_global_id(int index) const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.Variant.proxy_global_id)
  return _internal_proxy_global_id().Get(index);
}
inline void Variant::set_proxy_global_id(int index, ::uint64_t value) {
  _internal_mutable_proxy_global_id()->Set(index, value);
  // @@protoc_insertion_point(field_set:paraview_protobuf.Variant.proxy_global_id)
}
inline void Variant::add_proxy_global_id(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_proxy_global_id()
      ->InternalAddWithArena<const ::google::protobuf::MessageLite*>(
          internal_visibility(), this, value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_add:paraview_protobuf.Variant.proxy_global_id)
}
inline const ::google::protobuf::RepeatedField<::uint64_t>& Variant::proxy_global_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:paraview_protobuf.Variant.proxy_global_id)
  return _internal_proxy_global_id();
}
inline ::google::protobuf::RepeatedField<::uint64_t>* PROTOBUF_NONNULL Variant::mutable_proxy_global_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_mutable_list:paraview_protobuf.Variant.proxy_global_id)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_proxy_global_id();
}
inline const ::google::protobuf::RepeatedField<::uint64_t>&
Variant::_internal_proxy_global_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.proxy_global_id_;
}
inline ::google::protobuf::RepeatedField<::uint64_t>* PROTOBUF_NONNULL
Variant::_internal_mutable_proxy_global_id() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.proxy_global_id_;
}

// repeated int32 port_number = 7;
inline int Variant::_internal_port_number_size() const {
  return _internal_port_number().size();
}
inline int Variant::port_number_size() const {
  return _internal_port_number_size();
}
inline void Variant::clear_port_number() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.port_number_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int32_t Variant::port_number(int index) const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.Variant.port_number)
  return _internal_port_number().Get(index);
}
inline void Variant::set_port_number(int index, ::int32_t value) {
  _internal_mutable_port_number()->Set(index, value);
  // @@protoc_insertion_point(field_set:paraview_protobuf.Variant.port_number)
}
inline void Variant::add_port_number(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_port_number()
      ->InternalAddWithArena<const ::google::protobuf::MessageLite*>(
          internal_visibility(), this, value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_add:paraview_protobuf.Variant.port_number)
}
inline const ::google::protobuf::RepeatedField<::int32_t>& Variant::port_number() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:paraview_protobuf.Variant.port_number)
  return _internal_port_number();
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL Variant::mutable_port_number()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_mutable_list:paraview_protobuf.Variant.port_number)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_port_number();
}
inline const ::google::protobuf::RepeatedField<::int32_t>&
Variant::_internal_port_number() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.port_number_;
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL
Variant::_internal_mutable_port_number() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.port_number_;
}

// repeated string txt = 8;
inline int Variant::_internal_txt_size() const {
  return _internal_txt().size();
}
inline int Variant::txt_size() const {
  return _internal_txt_size();
}
inline void Variant::clear_txt() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.txt_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::std::string* PROTOBUF_NONNULL Variant::add_txt()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_txt()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_add_mutable:paraview_protobuf.Variant.txt)
  return _s;
}
inline const ::std::string& Variant::txt(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.Variant.txt)
  return _internal_txt().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL Variant::mutable_txt(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.Variant.txt)
  return _internal_mutable_txt()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void Variant::set_txt(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(
      *_internal_mutable_txt()->Mutable(index),
      ::std::forward<Arg_>(value), args... );
  // @@protoc_insertion_point(field_set:paraview_protobuf.Variant.txt)
}
template <typename Arg_, typename... Args_>
inline void Variant::add_txt(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_txt(), ::std::forward<Arg_>(value),
      args... );
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_add:paraview_protobuf.Variant.txt)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& Variant::txt()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:paraview_protobuf.Variant.txt)
  return _internal_txt();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Variant::mutable_txt() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_mutable_list:paraview_protobuf.Variant.txt)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_txt();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
Variant::_internal_txt() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.txt_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Variant::_internal_mutable_txt() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.txt_;
}

// repeated bytes binary = 9;
inline int Variant::_internal_binary_size() const {
  return _internal_binary().size();
}
inline int Variant::binary_size() const {
  return _internal_binary_size();
}
inline void Variant::clear_binary() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.binary_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::std::string* PROTOBUF_NONNULL Variant::add_binary()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_binary()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_add_mutable:paraview_protobuf.Variant.binary)
  return _s;
}
inline const ::std::string& Variant::binary(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.Variant.binary)
  return _internal_binary().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL Variant::mutable_binary(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.Variant.binary)
  return _internal_mutable_binary()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void Variant::set_binary(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(
      *_internal_mutable_binary()->Mutable(index),
      ::std::forward<Arg_>(value), args... , ::google::protobuf::internal::BytesTag{});
  // @@protoc_insertion_point(field_set:paraview_protobuf.Variant.binary)
}
template <typename Arg_, typename... Args_>
inline void Variant::add_binary(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_binary(), ::std::forward<Arg_>(value),
      args... , ::google::protobuf::internal::BytesTag{});
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_add:paraview_protobuf.Variant.binary)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& Variant::binary()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:paraview_protobuf.Variant.binary)
  return _internal_binary();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Variant::mutable_binary() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_mutable_list:paraview_protobuf.Variant.binary)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_binary();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
Variant::_internal_binary() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.binary_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Variant::_internal_mutable_binary() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.binary_;
}

// -------------------------------------------------------------------

// VariantList

// repeated .paraview_protobuf.Variant variant = 1;
inline int VariantList::_internal_variant_size() const {
  return _internal_variant().size();
}
inline int VariantList::variant_size() const {
  return _internal_variant_size();
}
inline void VariantList::clear_variant() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.variant_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::paraview_protobuf::Variant& VariantList::variant(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.VariantList.variant)
  return _internal_variant().Get(index);
}
inline ::paraview_protobuf::Variant* PROTOBUF_NONNULL VariantList::mutable_variant(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.VariantList.variant)
  return _internal_mutable_variant()->Mutable(index);
}
inline ::paraview_protobuf::Variant* PROTOBUF_NONNULL VariantList::add_variant()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::paraview_protobuf::Variant* _add =
      _internal_mutable_variant()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:paraview_protobuf.VariantList.variant)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Variant>& VariantList::variant() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:paraview_protobuf.VariantList.variant)
  return _internal_variant();
}
inline ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Variant>* PROTOBUF_NONNULL
VariantList::mutable_variant() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:paraview_protobuf.VariantList.variant)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_variant();
}
inline const ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Variant>&
VariantList::_internal_variant() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.variant_;
}
inline ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Variant>* PROTOBUF_NONNULL
VariantList::_internal_mutable_variant() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.variant_;
}

// -------------------------------------------------------------------

// MessageCollection

// repeated .paraview_protobuf.Message item = 1;
inline int MessageCollection::_internal_item_size() const {
  return _internal_item().size();
}
inline int MessageCollection::item_size() const {
  return _internal_item_size();
}
inline void MessageCollection::clear_item() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.item_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::paraview_protobuf::Message& MessageCollection::item(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.MessageCollection.item)
  return _internal_item().Get(index);
}
inline ::paraview_protobuf::Message* PROTOBUF_NONNULL MessageCollection::mutable_item(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.MessageCollection.item)
  return _internal_mutable_item()->Mutable(index);
}
inline ::paraview_protobuf::Message* PROTOBUF_NONNULL MessageCollection::add_item()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::paraview_protobuf::Message* _add =
      _internal_mutable_item()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:paraview_protobuf.MessageCollection.item)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Message>& MessageCollection::item() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:paraview_protobuf.MessageCollection.item)
  return _internal_item();
}
inline ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Message>* PROTOBUF_NONNULL
MessageCollection::mutable_item() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:paraview_protobuf.MessageCollection.item)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_item();
}
inline const ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Message>&
MessageCollection::_internal_item() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.item_;
}
inline ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Message>* PROTOBUF_NONNULL
MessageCollection::_internal_mutable_item() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.item_;
}

// -------------------------------------------------------------------

// Message

// required uint64 global_id = 1;
inline bool Message::has_global_id() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void Message::clear_global_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.global_id_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::uint64_t Message::global_id() const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.Message.global_id)
  return _internal_global_id();
}
inline void Message::set_global_id(::uint64_t value) {
  _internal_set_global_id(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:paraview_protobuf.Message.global_id)
}
inline ::uint64_t Message::_internal_global_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.global_id_;
}
inline void Message::_internal_set_global_id(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.global_id_ = value;
}

// required uint32 location = 2;
inline bool Message::has_location() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void Message::clear_location() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.location_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::uint32_t Message::location() const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.Message.location)
  return _internal_location();
}
inline void Message::set_location(::uint32_t value) {
  _internal_set_location(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:paraview_protobuf.Message.location)
}
inline ::uint32_t Message::_internal_location() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.location_;
}
inline void Message::_internal_set_location(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.location_ = value;
}

// optional bool share_only = 3 [default = false];
inline bool Message::has_share_only() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void Message::clear_share_only() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.share_only_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline bool Message::share_only() const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.Message.share_only)
  return _internal_share_only();
}
inline void Message::set_share_only(bool value) {
  _internal_set_share_only(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:paraview_protobuf.Message.share_only)
}
inline bool Message::_internal_share_only() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.share_only_;
}
inline void Message::_internal_set_share_only(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.share_only_ = value;
}

// optional bool req_def = 4 [default = false];
inline bool Message::has_req_def() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  return value;
}
inline void Message::clear_req_def() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.req_def_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline bool Message::req_def() const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.Message.req_def)
  return _internal_req_def();
}
inline void Message::set_req_def(bool value) {
  _internal_set_req_def(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:paraview_protobuf.Message.req_def)
}
inline bool Message::_internal_req_def() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.req_def_;
}
inline void Message::_internal_set_req_def(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.req_def_ = value;
}

// optional uint32 client_id = 5 [default = 0];
inline bool Message::has_client_id() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  return value;
}
inline void Message::clear_client_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.client_id_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint32_t Message::client_id() const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.Message.client_id)
  return _internal_client_id();
}
inline void Message::set_client_id(::uint32_t value) {
  _internal_set_client_id(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:paraview_protobuf.Message.client_id)
}
inline ::uint32_t Message::_internal_client_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.client_id_;
}
inline void Message::_internal_set_client_id(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.client_id_ = value;
}

// -------------------------------------------------------------------

// DefinitionHeader

// -------------------------------------------------------------------

// ProxyState_SubProxy

// required string name = 1;
inline bool ProxyState_SubProxy::has_name() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void ProxyState_SubProxy::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& ProxyState_SubProxy::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ProxyState.SubProxy.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ProxyState_SubProxy::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:paraview_protobuf.ProxyState.SubProxy.name)
}
inline ::std::string* PROTOBUF_NONNULL ProxyState_SubProxy::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.ProxyState.SubProxy.name)
  return _s;
}
inline const ::std::string& ProxyState_SubProxy::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void ProxyState_SubProxy::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ProxyState_SubProxy::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ProxyState_SubProxy::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:paraview_protobuf.ProxyState.SubProxy.name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void ProxyState_SubProxy::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:paraview_protobuf.ProxyState.SubProxy.name)
}

// required uint32 global_id = 2;
inline bool ProxyState_SubProxy::has_global_id() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void ProxyState_SubProxy::clear_global_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.global_id_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::uint32_t ProxyState_SubProxy::global_id() const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ProxyState.SubProxy.global_id)
  return _internal_global_id();
}
inline void ProxyState_SubProxy::set_global_id(::uint32_t value) {
  _internal_set_global_id(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:paraview_protobuf.ProxyState.SubProxy.global_id)
}
inline ::uint32_t ProxyState_SubProxy::_internal_global_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.global_id_;
}
inline void ProxyState_SubProxy::_internal_set_global_id(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.global_id_ = value;
}

// -------------------------------------------------------------------

// ProxyState_Annotation

// required string key = 1;
inline bool ProxyState_Annotation::has_key() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void ProxyState_Annotation::clear_key() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.key_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& ProxyState_Annotation::key() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ProxyState.Annotation.key)
  return _internal_key();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ProxyState_Annotation::set_key(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.key_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:paraview_protobuf.ProxyState.Annotation.key)
}
inline ::std::string* PROTOBUF_NONNULL ProxyState_Annotation::mutable_key()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_key();
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.ProxyState.Annotation.key)
  return _s;
}
inline const ::std::string& ProxyState_Annotation::_internal_key() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.key_.Get();
}
inline void ProxyState_Annotation::_internal_set_key(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.key_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ProxyState_Annotation::_internal_mutable_key() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.key_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ProxyState_Annotation::release_key() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:paraview_protobuf.ProxyState.Annotation.key)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.key_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.key_.Set("", GetArena());
  }
  return released;
}
inline void ProxyState_Annotation::set_allocated_key(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.key_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.key_.IsDefault()) {
    _impl_.key_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:paraview_protobuf.ProxyState.Annotation.key)
}

// required string value = 2;
inline bool ProxyState_Annotation::has_value() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void ProxyState_Annotation::clear_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.value_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& ProxyState_Annotation::value() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ProxyState.Annotation.value)
  return _internal_value();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ProxyState_Annotation::set_value(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.value_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:paraview_protobuf.ProxyState.Annotation.value)
}
inline ::std::string* PROTOBUF_NONNULL ProxyState_Annotation::mutable_value()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_value();
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.ProxyState.Annotation.value)
  return _s;
}
inline const ::std::string& ProxyState_Annotation::_internal_value() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.value_.Get();
}
inline void ProxyState_Annotation::_internal_set_value(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.value_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ProxyState_Annotation::_internal_mutable_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.value_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ProxyState_Annotation::release_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:paraview_protobuf.ProxyState.Annotation.value)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.value_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.value_.Set("", GetArena());
  }
  return released;
}
inline void ProxyState_Annotation::set_allocated_value(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.value_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.value_.IsDefault()) {
    _impl_.value_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:paraview_protobuf.ProxyState.Annotation.value)
}

// -------------------------------------------------------------------

// ProxyState_UserData

// required string key = 1;
inline bool ProxyState_UserData::has_key() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void ProxyState_UserData::clear_key() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.key_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& ProxyState_UserData::key() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ProxyState.UserData.key)
  return _internal_key();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ProxyState_UserData::set_key(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.key_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:paraview_protobuf.ProxyState.UserData.key)
}
inline ::std::string* PROTOBUF_NONNULL ProxyState_UserData::mutable_key()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_key();
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.ProxyState.UserData.key)
  return _s;
}
inline const ::std::string& ProxyState_UserData::_internal_key() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.key_.Get();
}
inline void ProxyState_UserData::_internal_set_key(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.key_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ProxyState_UserData::_internal_mutable_key() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.key_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ProxyState_UserData::release_key() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:paraview_protobuf.ProxyState.UserData.key)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.key_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.key_.Set("", GetArena());
  }
  return released;
}
inline void ProxyState_UserData::set_allocated_key(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.key_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.key_.IsDefault()) {
    _impl_.key_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:paraview_protobuf.ProxyState.UserData.key)
}

// repeated .paraview_protobuf.Variant variant = 2;
inline int ProxyState_UserData::_internal_variant_size() const {
  return _internal_variant().size();
}
inline int ProxyState_UserData::variant_size() const {
  return _internal_variant_size();
}
inline void ProxyState_UserData::clear_variant() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.variant_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::paraview_protobuf::Variant& ProxyState_UserData::variant(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ProxyState.UserData.variant)
  return _internal_variant().Get(index);
}
inline ::paraview_protobuf::Variant* PROTOBUF_NONNULL ProxyState_UserData::mutable_variant(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.ProxyState.UserData.variant)
  return _internal_mutable_variant()->Mutable(index);
}
inline ::paraview_protobuf::Variant* PROTOBUF_NONNULL ProxyState_UserData::add_variant()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::paraview_protobuf::Variant* _add =
      _internal_mutable_variant()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:paraview_protobuf.ProxyState.UserData.variant)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Variant>& ProxyState_UserData::variant() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:paraview_protobuf.ProxyState.UserData.variant)
  return _internal_variant();
}
inline ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Variant>* PROTOBUF_NONNULL
ProxyState_UserData::mutable_variant() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:paraview_protobuf.ProxyState.UserData.variant)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_variant();
}
inline const ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Variant>&
ProxyState_UserData::_internal_variant() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.variant_;
}
inline ::google::protobuf::RepeatedPtrField<::paraview_protobuf::Variant>* PROTOBUF_NONNULL
ProxyState_UserData::_internal_mutable_variant() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.variant_;
}

// -------------------------------------------------------------------

// ProxyState_Property

// required string name = 1;
inline bool ProxyState_Property::has_name() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void ProxyState_Property::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& ProxyState_Property::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ProxyState.Property.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ProxyState_Property::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:paraview_protobuf.ProxyState.Property.name)
}
inline ::std::string* PROTOBUF_NONNULL ProxyState_Property::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.ProxyState.Property.name)
  return _s;
}
inline const ::std::string& ProxyState_Property::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void ProxyState_Property::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ProxyState_Property::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ProxyState_Property::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:paraview_protobuf.ProxyState.Property.name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void ProxyState_Property::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:paraview_protobuf.ProxyState.Property.name)
}

// optional .paraview_protobuf.Variant value = 2;
inline bool ProxyState_Property::has_value() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.value_ != nullptr);
  return value;
}
inline void ProxyState_Property::clear_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.value_ != nullptr) _impl_.value_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::paraview_protobuf::Variant& ProxyState_Property::_internal_value() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::paraview_protobuf::Variant* p = _impl_.value_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::paraview_protobuf::Variant>(&::paraview_protobuf::Variant_globals_);
}
inline const ::paraview_protobuf::Variant& ProxyState_Property::value() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ProxyState.Property.value)
  return _internal_value();
}
inline void ProxyState_Property::unsafe_arena_set_allocated_value(
    ::paraview_protobuf::Variant* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.value_);
  }
  _impl_.value_ = reinterpret_cast<::paraview_protobuf::Variant*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:paraview_protobuf.ProxyState.Property.value)
}
inline ::paraview_protobuf::Variant* PROTOBUF_NULLABLE ProxyState_Property::release_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::paraview_protobuf::Variant* released = _impl_.value_;
  _impl_.value_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::paraview_protobuf::Variant* PROTOBUF_NULLABLE ProxyState_Property::unsafe_arena_release_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:paraview_protobuf.ProxyState.Property.value)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::paraview_protobuf::Variant* temp = _impl_.value_;
  _impl_.value_ = nullptr;
  return temp;
}
inline ::paraview_protobuf::Variant* PROTOBUF_NONNULL ProxyState_Property::_internal_mutable_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.value_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::paraview_protobuf::Variant>(GetArena());
    _impl_.value_ = reinterpret_cast<::paraview_protobuf::Variant*>(p);
  }
  return _impl_.value_;
}
inline ::paraview_protobuf::Variant* PROTOBUF_NONNULL ProxyState_Property::mutable_value()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::paraview_protobuf::Variant* _msg = _internal_mutable_value();
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.ProxyState.Property.value)
  return _msg;
}
inline void ProxyState_Property::set_allocated_value(::paraview_protobuf::Variant* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.value_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.value_ = reinterpret_cast<::paraview_protobuf::Variant*>(value);
  // @@protoc_insertion_point(field_set_allocated:paraview_protobuf.ProxyState.Property.value)
}

// repeated .paraview_protobuf.ProxyState.UserData user_data = 3;
inline int ProxyState_Property::_internal_user_data_size() const {
  return _internal_user_data().size();
}
inline int ProxyState_Property::user_data_size() const {
  return _internal_user_data_size();
}
inline void ProxyState_Property::clear_user_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.user_data_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::paraview_protobuf::ProxyState_UserData& ProxyState_Property::user_data(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ProxyState.Property.user_data)
  return _internal_user_data().Get(index);
}
inline ::paraview_protobuf::ProxyState_UserData* PROTOBUF_NONNULL ProxyState_Property::mutable_user_data(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.ProxyState.Property.user_data)
  return _internal_mutable_user_data()->Mutable(index);
}
inline ::paraview_protobuf::ProxyState_UserData* PROTOBUF_NONNULL ProxyState_Property::add_user_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::paraview_protobuf::ProxyState_UserData* _add =
      _internal_mutable_user_data()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:paraview_protobuf.ProxyState.Property.user_data)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::paraview_protobuf::ProxyState_UserData>& ProxyState_Property::user_data() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:paraview_protobuf.ProxyState.Property.user_data)
  return _internal_user_data();
}
inline ::google::protobuf::RepeatedPtrField<::paraview_protobuf::ProxyState_UserData>* PROTOBUF_NONNULL
ProxyState_Property::mutable_user_data() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:paraview_protobuf.ProxyState.Property.user_data)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_user_data();
}
inline const ::google::protobuf::RepeatedPtrField<::paraview_protobuf::ProxyState_UserData>&
ProxyState_Property::_internal_user_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.user_data_;
}
inline ::google::protobuf::RepeatedPtrField<::paraview_protobuf::ProxyState_UserData>* PROTOBUF_NONNULL
ProxyState_Property::_internal_mutable_user_data() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.user_data_;
}

// -------------------------------------------------------------------

// ProxyState

// -------------------------------------------------------------------

// PXMRegistrationState_Entry

// optional string group = 1;
inline bool PXMRegistrationState_Entry::has_group() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void PXMRegistrationState_Entry::clear_group() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.group_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& PXMRegistrationState_Entry::group() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.PXMRegistrationState.Entry.group)
  return _internal_group();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PXMRegistrationState_Entry::set_group(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.group_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:paraview_protobuf.PXMRegistrationState.Entry.group)
}
inline ::std::string* PROTOBUF_NONNULL PXMRegistrationState_Entry::mutable_group()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_group();
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.PXMRegistrationState.Entry.group)
  return _s;
}
inline const ::std::string& PXMRegistrationState_Entry::_internal_group() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.group_.Get();
}
inline void PXMRegistrationState_Entry::_internal_set_group(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.group_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PXMRegistrationState_Entry::_internal_mutable_group() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.group_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PXMRegistrationState_Entry::release_group() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:paraview_protobuf.PXMRegistrationState.Entry.group)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.group_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.group_.Set("", GetArena());
  }
  return released;
}
inline void PXMRegistrationState_Entry::set_allocated_group(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.group_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.group_.IsDefault()) {
    _impl_.group_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:paraview_protobuf.PXMRegistrationState.Entry.group)
}

// required string name = 2;
inline bool PXMRegistrationState_Entry::has_name() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void PXMRegistrationState_Entry::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& PXMRegistrationState_Entry::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.PXMRegistrationState.Entry.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PXMRegistrationState_Entry::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:paraview_protobuf.PXMRegistrationState.Entry.name)
}
inline ::std::string* PROTOBUF_NONNULL PXMRegistrationState_Entry::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.PXMRegistrationState.Entry.name)
  return _s;
}
inline const ::std::string& PXMRegistrationState_Entry::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void PXMRegistrationState_Entry::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PXMRegistrationState_Entry::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PXMRegistrationState_Entry::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:paraview_protobuf.PXMRegistrationState.Entry.name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void PXMRegistrationState_Entry::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:paraview_protobuf.PXMRegistrationState.Entry.name)
}

// required uint64 global_id = 3;
inline bool PXMRegistrationState_Entry::has_global_id() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void PXMRegistrationState_Entry::clear_global_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.global_id_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t PXMRegistrationState_Entry::global_id() const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.PXMRegistrationState.Entry.global_id)
  return _internal_global_id();
}
inline void PXMRegistrationState_Entry::set_global_id(::uint64_t value) {
  _internal_set_global_id(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:paraview_protobuf.PXMRegistrationState.Entry.global_id)
}
inline ::uint64_t PXMRegistrationState_Entry::_internal_global_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.global_id_;
}
inline void PXMRegistrationState_Entry::_internal_set_global_id(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.global_id_ = value;
}

// -------------------------------------------------------------------

// PXMRegistrationState

// -------------------------------------------------------------------

// ProxyDefinitionState_ProxyXMLDefinition

// required string group = 1;
inline bool ProxyDefinitionState_ProxyXMLDefinition::has_group() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void ProxyDefinitionState_ProxyXMLDefinition::clear_group() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.group_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& ProxyDefinitionState_ProxyXMLDefinition::group() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition.group)
  return _internal_group();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ProxyDefinitionState_ProxyXMLDefinition::set_group(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.group_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition.group)
}
inline ::std::string* PROTOBUF_NONNULL ProxyDefinitionState_ProxyXMLDefinition::mutable_group()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_group();
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition.group)
  return _s;
}
inline const ::std::string& ProxyDefinitionState_ProxyXMLDefinition::_internal_group() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.group_.Get();
}
inline void ProxyDefinitionState_ProxyXMLDefinition::_internal_set_group(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.group_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ProxyDefinitionState_ProxyXMLDefinition::_internal_mutable_group() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.group_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ProxyDefinitionState_ProxyXMLDefinition::release_group() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition.group)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.group_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.group_.Set("", GetArena());
  }
  return released;
}
inline void ProxyDefinitionState_ProxyXMLDefinition::set_allocated_group(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.group_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.group_.IsDefault()) {
    _impl_.group_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition.group)
}

// required string name = 2;
inline bool ProxyDefinitionState_ProxyXMLDefinition::has_name() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void ProxyDefinitionState_ProxyXMLDefinition::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& ProxyDefinitionState_ProxyXMLDefinition::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ProxyDefinitionState_ProxyXMLDefinition::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition.name)
}
inline ::std::string* PROTOBUF_NONNULL ProxyDefinitionState_ProxyXMLDefinition::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition.name)
  return _s;
}
inline const ::std::string& ProxyDefinitionState_ProxyXMLDefinition::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void ProxyDefinitionState_ProxyXMLDefinition::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ProxyDefinitionState_ProxyXMLDefinition::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ProxyDefinitionState_ProxyXMLDefinition::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition.name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void ProxyDefinitionState_ProxyXMLDefinition::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition.name)
}

// required string xml = 3;
inline bool ProxyDefinitionState_ProxyXMLDefinition::has_xml() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void ProxyDefinitionState_ProxyXMLDefinition::clear_xml() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.xml_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::std::string& ProxyDefinitionState_ProxyXMLDefinition::xml() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition.xml)
  return _internal_xml();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ProxyDefinitionState_ProxyXMLDefinition::set_xml(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.xml_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition.xml)
}
inline ::std::string* PROTOBUF_NONNULL ProxyDefinitionState_ProxyXMLDefinition::mutable_xml()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_xml();
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition.xml)
  return _s;
}
inline const ::std::string& ProxyDefinitionState_ProxyXMLDefinition::_internal_xml() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.xml_.Get();
}
inline void ProxyDefinitionState_ProxyXMLDefinition::_internal_set_xml(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.xml_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ProxyDefinitionState_ProxyXMLDefinition::_internal_mutable_xml() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.xml_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ProxyDefinitionState_ProxyXMLDefinition::release_xml() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition.xml)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.xml_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.xml_.Set("", GetArena());
  }
  return released;
}
inline void ProxyDefinitionState_ProxyXMLDefinition::set_allocated_xml(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.xml_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.xml_.IsDefault()) {
    _impl_.xml_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:paraview_protobuf.ProxyDefinitionState.ProxyXMLDefinition.xml)
}

// -------------------------------------------------------------------

// ProxyDefinitionState

// -------------------------------------------------------------------

// PullRequest

// -------------------------------------------------------------------

// ProxySelectionModelState

// -------------------------------------------------------------------

// LinkState_LinkDescription

// required uint32 proxy = 1;
inline bool LinkState_LinkDescription::has_proxy() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void LinkState_LinkDescription::clear_proxy() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.proxy_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::uint32_t LinkState_LinkDescription::proxy() const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.LinkState.LinkDescription.proxy)
  return _internal_proxy();
}
inline void LinkState_LinkDescription::set_proxy(::uint32_t value) {
  _internal_set_proxy(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:paraview_protobuf.LinkState.LinkDescription.proxy)
}
inline ::uint32_t LinkState_LinkDescription::_internal_proxy() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.proxy_;
}
inline void LinkState_LinkDescription::_internal_set_proxy(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.proxy_ = value;
}

// optional string property_name = 2;
inline bool LinkState_LinkDescription::has_property_name() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void LinkState_LinkDescription::clear_property_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.property_name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& LinkState_LinkDescription::property_name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.LinkState.LinkDescription.property_name)
  return _internal_property_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void LinkState_LinkDescription::set_property_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.property_name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:paraview_protobuf.LinkState.LinkDescription.property_name)
}
inline ::std::string* PROTOBUF_NONNULL LinkState_LinkDescription::mutable_property_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_property_name();
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.LinkState.LinkDescription.property_name)
  return _s;
}
inline const ::std::string& LinkState_LinkDescription::_internal_property_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.property_name_.Get();
}
inline void LinkState_LinkDescription::_internal_set_property_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.property_name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL LinkState_LinkDescription::_internal_mutable_property_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.property_name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE LinkState_LinkDescription::release_property_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:paraview_protobuf.LinkState.LinkDescription.property_name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.property_name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.property_name_.Set("", GetArena());
  }
  return released;
}
inline void LinkState_LinkDescription::set_allocated_property_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.property_name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.property_name_.IsDefault()) {
    _impl_.property_name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:paraview_protobuf.LinkState.LinkDescription.property_name)
}

// required .paraview_protobuf.LinkState.LinkDescription.Direction direction = 3;
inline bool LinkState_LinkDescription::has_direction() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void LinkState_LinkDescription::clear_direction() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.direction_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::paraview_protobuf::LinkState_LinkDescription_Direction LinkState_LinkDescription::direction() const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.LinkState.LinkDescription.direction)
  return _internal_direction();
}
inline void LinkState_LinkDescription::set_direction(::paraview_protobuf::LinkState_LinkDescription_Direction value) {
  _internal_set_direction(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:paraview_protobuf.LinkState.LinkDescription.direction)
}
inline ::paraview_protobuf::LinkState_LinkDescription_Direction LinkState_LinkDescription::_internal_direction() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::paraview_protobuf::LinkState_LinkDescription_Direction>(_impl_.direction_);
}
inline void LinkState_LinkDescription::_internal_set_direction(::paraview_protobuf::LinkState_LinkDescription_Direction value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);

                                          assert(::google::protobuf::internal::ValidateEnum(
                                              value, ::paraview_protobuf::LinkState_LinkDescription_Direction_internal_data_));
                                          _impl_.direction_ = value;
}

// -------------------------------------------------------------------

// LinkState

// -------------------------------------------------------------------

// MousePointer

// -------------------------------------------------------------------

// ChartViewBounds

// -------------------------------------------------------------------

// ClientsInformation_ClientInfo

// required uint32 user = 1;
inline bool ClientsInformation_ClientInfo::has_user() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void ClientsInformation_ClientInfo::clear_user() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.user_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::uint32_t ClientsInformation_ClientInfo::user() const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ClientsInformation.ClientInfo.user)
  return _internal_user();
}
inline void ClientsInformation_ClientInfo::set_user(::uint32_t value) {
  _internal_set_user(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:paraview_protobuf.ClientsInformation.ClientInfo.user)
}
inline ::uint32_t ClientsInformation_ClientInfo::_internal_user() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.user_;
}
inline void ClientsInformation_ClientInfo::_internal_set_user(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.user_ = value;
}

// optional string name = 2;
inline bool ClientsInformation_ClientInfo::has_name() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void ClientsInformation_ClientInfo::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& ClientsInformation_ClientInfo::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ClientsInformation.ClientInfo.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ClientsInformation_ClientInfo::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:paraview_protobuf.ClientsInformation.ClientInfo.name)
}
inline ::std::string* PROTOBUF_NONNULL ClientsInformation_ClientInfo::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:paraview_protobuf.ClientsInformation.ClientInfo.name)
  return _s;
}
inline const ::std::string& ClientsInformation_ClientInfo::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void ClientsInformation_ClientInfo::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ClientsInformation_ClientInfo::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ClientsInformation_ClientInfo::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:paraview_protobuf.ClientsInformation.ClientInfo.name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void ClientsInformation_ClientInfo::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:paraview_protobuf.ClientsInformation.ClientInfo.name)
}

// optional bool is_master = 3 [default = false];
inline bool ClientsInformation_ClientInfo::has_is_master() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void ClientsInformation_ClientInfo::clear_is_master() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.is_master_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline bool ClientsInformation_ClientInfo::is_master() const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ClientsInformation.ClientInfo.is_master)
  return _internal_is_master();
}
inline void ClientsInformation_ClientInfo::set_is_master(bool value) {
  _internal_set_is_master(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:paraview_protobuf.ClientsInformation.ClientInfo.is_master)
}
inline bool ClientsInformation_ClientInfo::_internal_is_master() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.is_master_;
}
inline void ClientsInformation_ClientInfo::_internal_set_is_master(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.is_master_ = value;
}

// optional bool follow_cam = 4 [default = false];
inline bool ClientsInformation_ClientInfo::has_follow_cam() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  return value;
}
inline void ClientsInformation_ClientInfo::clear_follow_cam() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.follow_cam_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline bool ClientsInformation_ClientInfo::follow_cam() const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ClientsInformation.ClientInfo.follow_cam)
  return _internal_follow_cam();
}
inline void ClientsInformation_ClientInfo::set_follow_cam(bool value) {
  _internal_set_follow_cam(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:paraview_protobuf.ClientsInformation.ClientInfo.follow_cam)
}
inline bool ClientsInformation_ClientInfo::_internal_follow_cam() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.follow_cam_;
}
inline void ClientsInformation_ClientInfo::_internal_set_follow_cam(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.follow_cam_ = value;
}

// optional bool disable_further_connections = 5 [default = false];
inline bool ClientsInformation_ClientInfo::has_disable_further_connections() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  return value;
}
inline void ClientsInformation_ClientInfo::clear_disable_further_connections() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.disable_further_connections_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline bool ClientsInformation_ClientInfo::disable_further_connections() const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ClientsInformation.ClientInfo.disable_further_connections)
  return _internal_disable_further_connections();
}
inline void ClientsInformation_ClientInfo::set_disable_further_connections(bool value) {
  _internal_set_disable_further_connections(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:paraview_protobuf.ClientsInformation.ClientInfo.disable_further_connections)
}
inline bool ClientsInformation_ClientInfo::_internal_disable_further_connections() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.disable_further_connections_;
}
inline void ClientsInformation_ClientInfo::_internal_set_disable_further_connections(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.disable_further_connections_ = value;
}

// optional uint32 connect_id = 6 [default = 0];
inline bool ClientsInformation_ClientInfo::has_connect_id() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000020U);
  return value;
}
inline void ClientsInformation_ClientInfo::clear_connect_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.connect_id_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::uint32_t ClientsInformation_ClientInfo::connect_id() const {
  // @@protoc_insertion_point(field_get:paraview_protobuf.ClientsInformation.ClientInfo.connect_id)
  return _internal_connect_id();
}
inline void ClientsInformation_ClientInfo::set_connect_id(::uint32_t value) {
  _internal_set_connect_id(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:paraview_protobuf.ClientsInformation.ClientInfo.connect_id)
}
inline ::uint32_t ClientsInformation_ClientInfo::_internal_connect_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.connect_id_;
}
inline void ClientsInformation_ClientInfo::_internal_set_connect_id(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.connect_id_ = value;
}

// -------------------------------------------------------------------

// ClientsInformation

// -------------------------------------------------------------------

// ChatMessage

// -------------------------------------------------------------------

// QtEvent

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace paraview_protobuf


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::paraview_protobuf::Variant_Type> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::paraview_protobuf::Variant_Type>() {
  return ::paraview_protobuf::Variant_Type_descriptor();
}
template <>
struct is_proto_enum<::paraview_protobuf::LinkState_LinkDescription_Direction> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::paraview_protobuf::LinkState_LinkDescription_Direction>() {
  return ::paraview_protobuf::LinkState_LinkDescription_Direction_descriptor();
}
template <>
struct is_proto_enum<::paraview_protobuf::MousePointer_Ratio> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::paraview_protobuf::MousePointer_Ratio>() {
  return ::paraview_protobuf::MousePointer_Ratio_descriptor();
}
template <>
struct is_proto_enum<::paraview_protobuf::QtEvent_QtEventType> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::paraview_protobuf::QtEvent_QtEventType>() {
  return ::paraview_protobuf::QtEvent_QtEventType_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // vtkPVMessage_2eproto_2epb_2eh
